@@ -20,7 +20,6 @@
 #include "scripting/abc.h"
 #include "audio.h"
 #include "input.h"
-#include "swf.h"
 #include "rendering.h"
 #include "compat.h"
 
@@ -33,24 +32,23 @@
 using namespace lightspark;
 using namespace std;
 
-InputThread::InputThread(SystemState* s):m_sys(s),terminated(false),threaded(false),
-	mutexListeners("Input listeners"),mutexDragged("Input dragged"),curDragged(NULL),currentMouseOver(NULL),lastMouseDownTarget(NULL),
+InputThread::InputThread(SystemState* s):m_sys(s),engineData(NULL),terminated(false),threaded(false),
+	curDragged(),currentMouseOver(),lastMouseDownTarget(),
 	dragLimit(NULL)
 {
 	LOG(LOG_INFO,_("Creating input thread"));
 }
 
-void InputThread::start(const EngineData* e)
+void InputThread::start(EngineData* e)
 {
-	GtkWidget* container=e->container;
-	gtk_widget_set_can_focus(container,True);
-	gtk_widget_add_events(container,GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK |
-					GDK_POINTER_MOTION_MASK | GDK_EXPOSURE_MASK);
-	g_signal_connect(G_OBJECT(container), "event", G_CALLBACK(worker), this);
+	engineData = e;
+	engineData->setInputHandler(sigc::mem_fun(this, &InputThread::worker));
 }
 
 InputThread::~InputThread()
 {
+	if(engineData)
+		engineData->removeInputHandler();
 	wait();
 }
 
@@ -59,52 +57,49 @@ void InputThread::wait()
 	if(terminated)
 		return;
 	if(threaded)
-		pthread_join(t,NULL);
+		t->join();
 	terminated=true;
 }
 
-//This is a GTK event handler and the gdk lock is already acquired
-gboolean InputThread::worker(GtkWidget *widget, GdkEvent *event, InputThread* th)
+//This is guarded gdk_threads_enter/leave
+bool InputThread::worker(GdkEvent *event)
 {
 	//Set sys to this SystemState
-	sys=th->m_sys;
+	setTLSSys(m_sys);
 	gboolean ret=FALSE;
 	switch(event->type)
 	{
 		case GDK_KEY_PRESS:
 		{
-			//cout << "key press" << endl;
+			//LOG(LOG_INFO, "key press");
 			switch(event->key.keyval)
 			{
 				case GDK_q:
-					if(th->m_sys->standalone)
-					{
-						th->m_sys->setShutdownFlag();
-						gtk_main_quit();
-					}
+					if(m_sys->standalone)
+						m_sys->setShutdownFlag();
 					break;
 				case GDK_p:
-					th->m_sys->showProfilingData=!th->m_sys->showProfilingData;
+					m_sys->showProfilingData=!m_sys->showProfilingData;
 					break;
 				case GDK_m:
-					if (!th->m_sys->audioManager->pluginLoaded())
+					if (!m_sys->audioManager->pluginLoaded())
 						break;
-					th->m_sys->audioManager->toggleMuteAll();
-					if(th->m_sys->audioManager->allMuted())
+					m_sys->audioManager->toggleMuteAll();
+					if(m_sys->audioManager->allMuted())
 						LOG(LOG_INFO, "All sounds muted");
 					else
 						LOG(LOG_INFO, "All sounds unmuted");
 					break;
 				case GDK_c:
-					if(th->m_sys->hasError())
+					if(m_sys->hasError())
 					{
 						GtkClipboard *clipboard;
 						clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
-						gtk_clipboard_set_text(clipboard, th->m_sys->getErrorCause().c_str(),
-								th->m_sys->getErrorCause().size());
+						gtk_clipboard_set_text(clipboard, m_sys->getErrorCause().c_str(),
+								m_sys->getErrorCause().size());
 						clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
-						gtk_clipboard_set_text(clipboard, th->m_sys->getErrorCause().c_str(),
-								th->m_sys->getErrorCause().size());
+						gtk_clipboard_set_text(clipboard, m_sys->getErrorCause().c_str(),
+								m_sys->getErrorCause().size());
 						LOG(LOG_INFO, "Copied error to clipboard");
 					}
 					else
@@ -119,41 +114,40 @@ gboolean InputThread::worker(GtkWidget *widget, GdkEvent *event, InputThread* th
 		case GDK_EXPOSE:
 		{
 			//Signal the renderThread
-			th->m_sys->getRenderThread()->draw(false);
+			m_sys->getRenderThread()->draw(false);
 			ret=TRUE;
 			break;
 		}
 		case GDK_BUTTON_PRESS:
 		{
 			//Grab focus, to receive keypresses
-			gtk_widget_grab_focus(widget);
 			if(event->button.button == 1)
-				th->handleMouseDown(event->button.x,event->button.y);
+				handleMouseDown(event->button.x,event->button.y);
 			ret=TRUE;
 			break;
 		}
 		case GDK_2BUTTON_PRESS:
 		{
 			if(event->button.button == 1)
-				th->handleMouseDoubleClick(event->button.x,event->button.y);
+				handleMouseDoubleClick(event->button.x,event->button.y);
 			ret=TRUE;
 			break;
 		}
 		case GDK_BUTTON_RELEASE:
 		{
-			th->handleMouseUp(event->button.x,event->button.y);
+			handleMouseUp(event->button.x,event->button.y);
 			ret=TRUE;
 			break;
 		}
 		case GDK_MOTION_NOTIFY:
 		{
-			th->handleMouseMove(event->motion.x,event->motion.y);
+			handleMouseMove(event->motion.x,event->motion.y);
 			ret=TRUE;
 			break;
 		}
 		default:
 //#ifdef EXPENSIVE_DEBUG
-//			cout << "GDKTYPE " << event->type << endl;
+//			LOG(LOG_INFO, "GDKTYPE " << event->type);
 //#endif
 			break;
 	}
@@ -174,7 +168,7 @@ _NR<InteractiveObject> InputThread::getMouseTarget(uint32_t x, uint32_t y, Displ
 		return NullRef;
 	}
 	assert(maskStack.empty());
-	assert(selected!=NULL); /* atleast we hit the stage */
+	assert(selected); /* atleast we hit the stage */
 	assert_and_throw(selected->getClass()->isSubClass(Class<InteractiveObject>::getClass()));
 	return selected;
 }
@@ -231,11 +225,11 @@ void InputThread::handleMouseMove(uint32_t x, uint32_t y)
 	mousePos = Vector2(x,y);
 	Locker locker2(mutexDragged);
 	// Handle current drag operation
-	if(curDragged != NULL)
+	if(curDragged)
 	{
 		Vector2f local;
 		_NR<DisplayObjectContainer> parent = curDragged->getParent();
-		if(parent == NULL)
+		if(!parent)
 		{
 			stopDrag(curDragged.getPtr());
 			return;
