@@ -20,25 +20,27 @@
 #ifndef RENDERING_H
 #define RENDERING_H
 
+#include "lsopengl.h"
 #include "timer.h"
-#include <FTGL/ftgl.h>
 
 namespace lightspark
 {
 
+enum VertexAttrib { VERTEX_ATTRIB=0, COLOR_ATTRIB, TEXCOORD_ATTRIB};
+enum LSGL_MATRIX {LSGL_PROJECTION=0, LSGL_MODELVIEW};
+
 class RenderThread: public ITickJob
 {
 friend class DisplayObject;
 private:
 	SystemState* m_sys;
-	pthread_t t;
+	Thread* t;
 	enum STATUS { CREATED=0, STARTED, TERMINATED };
 	STATUS status;
-	static void* sdl_worker(RenderThread*);
-#ifdef COMPILE_PLUGIN
-	NPAPI_params* npapi_params;
-	static void* gtkplug_worker(RenderThread*);
-#endif
+
+	EngineData* engineData;
+	static void worker(RenderThread*);
+
 	void commonGLInit(int width, int height);
 	void commonGLResize();
 	void commonGLDeinit();
@@ -73,7 +75,7 @@ private:
 	void handleNewTexture();
 	void finalizeUpload();
 	void handleUpload();
-	sem_t event;
+	Semaphore event;
 	std::string fontPath;
 	uint32_t newWidth;
 	uint32_t newHeight;
@@ -82,13 +84,19 @@ private:
 	int offsetX;
 	int offsetY;
 
-#ifndef WIN32
+#ifndef _WIN32
 	Display* mDisplay;
+#ifndef ENABLE_GLES2
 	GLXFBConfig mFBConfig;
 	GLXContext mContext;
+#else
+	EGLContext mEGLContext;
+	EGLConfig mEGLConfig;
+#endif
 	Window mWindow;
 #endif
-	uint64_t time_s, time_d;
+	Glib::TimeVal time_s, time_d;
+	static Glib::TimeVal FPS_time;
 
 	bool loadShaderPrograms();
 	bool tempBufferAcquired;
@@ -104,7 +112,8 @@ private:
 	/*
 		Common code to handle the core of the rendering
 	*/
-	void coreRendering(FTFont& font);
+	void coreRendering();
+	void plotProfilingData();
 	Semaphore initialized;
 	class MaskData
 	{
@@ -114,19 +123,26 @@ private:
 		MaskData(DisplayObject* _d, const MATRIX& _m):d(_d),m(_m){}
 	};
 	std::vector<MaskData> maskStack;
+
+	static void SizeAllocateCallback(GtkWidget* widget, GdkRectangle* allocation, gpointer data);
 public:
 	RenderThread(SystemState* s);
 	~RenderThread();
-	void start(ENGINE e,void* param);
+	/**
+	   The EngineData object must survive for the whole life of this RenderThread
+	*/
+	void start(EngineData* data);
 	/*
 	   The stop function should be call on exit even if the thread is not started
 	*/
 	void stop();
 	void wait();
 	void draw(bool force);
+
 	//The calling context MUST call this function with the transformation matrix ready
-	void glAcquireTempBuffer(number_t xmin, number_t xmax, number_t ymin, number_t ymax);
-	void glBlitTempBuffer(number_t xmin, number_t xmax, number_t ymin, number_t ymax);
+	//void acquireTempBuffer(number_t xmin, number_t xmax, number_t ymin, number_t ymax);
+	//void blitTempBuffer(number_t xmin, number_t xmax, number_t ymin, number_t ymax);
+
 	/**
 		Allocates a chunk from the shared texture
 	*/
@@ -138,7 +154,7 @@ public:
 	/**
 		Render a quad of given size using the given chunk
 	*/
-	void renderTextured(const TextureChunk& chunk, uint32_t x, uint32_t y, uint32_t w, uint32_t h);
+	void renderTextured(const TextureChunk& chunk, int32_t x, int32_t y, uint32_t w, uint32_t h);
 	/**
 		Load the given data in the given texture chunk
 	*/
@@ -155,7 +171,7 @@ public:
 	*/
 	void pushMask(DisplayObject* d, const MATRIX& m)
 	{
-		maskStack.emplace_back(d,m);
+		maskStack.push_back(MaskData(d,m));
 	}
 	/**
 	  	Remove the last pushed mask
@@ -187,9 +203,28 @@ public:
 	uint32_t windowWidth;
 	uint32_t windowHeight;
 	bool hasNPOTTextures;
-	int fragmentTexScaleUniform;
+	GLint fragmentTexScaleUniform;
+	GLint yuvUniform;
+	GLint maskUniform;
+	GLint alphaUniform;
+	GLint directUniform;
+	GLint projectionMatrixUniform;
+	GLint modelviewMatrixUniform;
+
+	void renderErrorPage(RenderThread *rt, bool standalone);
+
+	cairo_t *cairoTextureContext;
+	cairo_surface_t *cairoTextureSurface;
+	uint8_t *cairoTextureData;
+	GLuint cairoTextureID;
+	cairo_t* getCairoContext(int w, int h);
+	void mapCairoTexture(int w, int h);
+	void renderText(cairo_t *cr, const char *text, int x, int y);
+	void setMatrixUniform(LSGL_MATRIX m) const;
+	static bool handleGLErrors();
 };
 
+RenderThread* getRenderThread();
+
 };
-extern TLSDATA lightspark::RenderThread* rt DLL_PUBLIC;
 #endif
