@@ -48,7 +48,6 @@ SecurityManager::SecurityManager():
 	sandboxTitles[2] = "local-with-networking";
 	sandboxTitles[3] = "local-trusted";
 
-	sem_init(&mutex,0,1);
 	//== Lock initialized
 }
 
@@ -59,9 +58,8 @@ SecurityManager::SecurityManager():
  */
 SecurityManager::~SecurityManager()
 {
-	sem_wait(&mutex);
-	//-- Lock acquired
-	
+	RecMutex::Lock l(mutex);
+
 	URLPFileMapIt i = pendingURLPFiles.begin();
 	for(; i != pendingURLPFiles.end(); ++i)
 		delete (*i).second;
@@ -69,9 +67,6 @@ SecurityManager::~SecurityManager()
 	i = loadedURLPFiles.begin();
 	for(;i != loadedURLPFiles.end(); ++i)
 		delete (*i).second;
-	
-	//== Destroy lock
-	sem_destroy(&mutex);
 }
 
 /**
@@ -103,19 +98,16 @@ PolicyFile* SecurityManager::addPolicyFile(const URLInfo& url)
  */
 URLPolicyFile* SecurityManager::addURLPolicyFile(const URLInfo& url)
 {
-	sem_wait(&mutex);
-	//-- Lock acquired
-	
+	RecMutex::Lock l(mutex);
+
 	URLPolicyFile* file = new URLPolicyFile(url);
 	if(file->isValid())
 	{
-		LOG(LOG_NO_INFO, 
+		LOG(LOG_INFO, 
 				_("SECURITY: Added URL policy file is valid, adding to URL policy file list (") << url << ")");
 		pendingURLPFiles.insert(URLPFilePair(url.getHostname(), file));
 	}
-	
-	//++ Release lock
-	sem_post(&mutex);
+
 	return file;
 }
 
@@ -130,8 +122,7 @@ URLPolicyFile* SecurityManager::addURLPolicyFile(const URLInfo& url)
  */
 URLPolicyFile* SecurityManager::getURLPolicyFileByURL(const URLInfo& url)
 {
-	sem_wait(&mutex);
-	//-- Lock acquired
+	RecMutex::Lock l(mutex);
 
 	URLPFileMapConstItPair range = loadedURLPFiles.equal_range(url.getHostname());
 	URLPFileMapConstIt i = range.first;
@@ -141,10 +132,8 @@ URLPolicyFile* SecurityManager::getURLPolicyFileByURL(const URLInfo& url)
 	{
 		if((*i).second->getOriginalURL() == url)
 		{
-			LOG(LOG_NO_INFO, _("SECURITY: URL policy file found in loaded list (") << url << ")");
+			LOG(LOG_INFO, _("SECURITY: URL policy file found in loaded list (") << url << ")");
 
-			//++ Release lock
-			sem_post(&mutex);
 			return (*i).second;
 		}
 	}
@@ -156,16 +145,12 @@ URLPolicyFile* SecurityManager::getURLPolicyFileByURL(const URLInfo& url)
 	{
 		if((*i).second->getOriginalURL() == url)
 		{
-			LOG(LOG_NO_INFO, _("SECURITY: URL policy file found in pending list (") << url << ")");
+			LOG(LOG_INFO, _("SECURITY: URL policy file found in pending list (") << url << ")");
 
-			//++ Release lock
-			sem_post(&mutex);
 			return (*i).second;
 		}
 	}
 
-	//++ Release lock
-	sem_post(&mutex);
 	return NULL;
 }
 
@@ -179,12 +164,11 @@ URLPolicyFile* SecurityManager::getURLPolicyFileByURL(const URLInfo& url)
  */
 void SecurityManager::loadPolicyFile(URLPolicyFile* file)
 {
-	sem_wait(&mutex);
-	//-- Lock acquired
+	RecMutex::Lock l(mutex);
 
 	if(pendingURLPFiles.count(file->getURL().getHostname()) > 0)
 	{
-		LOG(LOG_NO_INFO, _("SECURITY: Loading policy file (") << file->getURL() << ")");
+		LOG(LOG_INFO, _("SECURITY: Loading policy file (") << file->getURL() << ")");
 		file->load();
 
 		URLPFileMapItPair range = pendingURLPFiles.equal_range(file->getURL().getHostname());
@@ -199,9 +183,6 @@ void SecurityManager::loadPolicyFile(URLPolicyFile* file)
 			}
 		}
 	}
-
-	//++ Release lock
-	sem_post(&mutex);
 }
 
 /**
@@ -228,27 +209,22 @@ URLPFileList* SecurityManager::searchURLPolicyFiles(const URLInfo& url, bool loa
 		master = addURLPolicyFile(masterURL);
 
 	if(loadPendingPolicies)
-		sys->securityManager->loadPolicyFile(master);
-
-	sem_wait(&mutex);
-	//-- Lock acquired
+		getSys()->securityManager->loadPolicyFile(master);
 
+	RecMutex::Lock l(mutex);
 	//Check if the master policy file is loaded.
 	//If another user-added relevant policy file is already loaded, 
 	//it's master will have already been loaded too (to check if it is allowed).
 	//So IF any relevant policy file is loaded already, then the master will be too.
 	if(master->isLoaded() && master->isValid())
 	{
-		LOG(LOG_NO_INFO, _("SECURITY: Master policy file is loaded and valid (") << url << ")");
+		LOG(LOG_INFO, _("SECURITY: Master policy file is loaded and valid (") << url << ")");
 
 		PolicySiteControl::METAPOLICY siteControl = master->getSiteControl()->getPermittedPolicies();
 		//Master defines no policy files are allowed at all
 		if(siteControl == PolicySiteControl::NONE)
 		{
-			LOG(LOG_NO_INFO, _("SECURITY: DISALLOWED: Master policy file disallows policy files"));
-
-			//++ Release lock
-			sem_post(&mutex);
+			LOG(LOG_INFO, _("SECURITY: DISALLOWED: Master policy file disallows policy files"));
 			return NULL;
 		}
 
@@ -257,7 +233,7 @@ URLPFileList* SecurityManager::searchURLPolicyFiles(const URLInfo& url, bool loa
 		//Non-master policy files are allowed
 		if(siteControl != PolicySiteControl::MASTER_ONLY)
 		{
-			LOG(LOG_NO_INFO, _("SECURITY: Searching for loaded non-master policy files (") <<
+			LOG(LOG_INFO, _("SECURITY: Searching for loaded non-master policy files (") <<
 					loadedURLPFiles.count(url.getHostname()) << ")");
 
 			URLPFileMapConstItPair range = loadedURLPFiles.equal_range(url.getHostname());
@@ -272,28 +248,23 @@ URLPFileList* SecurityManager::searchURLPolicyFiles(const URLInfo& url, bool loa
 			//And check the pending policy files next (if we are allowed to)
 			if(loadPendingPolicies)
 			{
-				LOG(LOG_NO_INFO, _("SECURITY: Searching for and loading pending non-master policy files (") <<
+				LOG(LOG_INFO, _("SECURITY: Searching for and loading pending non-master policy files (") <<
 						pendingURLPFiles.count(url.getHostname()) << ")");
 
 				range = pendingURLPFiles.equal_range(url.getHostname());
 				i = range.first;
 				for(; i != range.second; ++i)
 				{
-					//++ Release lock
-					sem_post(&mutex);
+					l.release();
 					//NOTE: loadPolicyFile() will change pendingURLPFiles, erasing & moving to loadURLPFiles
-					sys->securityManager->loadPolicyFile((*i).second);
-					sem_wait(&mutex);
-					//-- Lock acquired
+					getSys()->securityManager->loadPolicyFile((*i).second);
+					l.acquire();
 
 					result->push_back((*i).second);
 				}
 			}
 		}
 	}
-
-	//++ Release lock
-	sem_post(&mutex);
 	return result;
 }
 
@@ -310,16 +281,13 @@ bool SecurityManager::evaluateSandbox(SANDBOXTYPE sandbox, int allowedSandboxes)
 }
 
 /**
- * \brief Evaluates an URL to see if it is allowed by the various security features.
+ * \brief Evaluates an URL static properties to see if it is allowed by the various security features.
  *
  * This first checks if the current sandbox is allowed access to the URL.
  * Next the port is checked to see if isn't restricted.
  * Then, if requested, the URL is checked to see if it doesn't point to a resource 
  * above the current directory in the directory hierarchy (only applicable to local URLs).
- * And finally the URL policy files are checked to see if they allow the player 
- * to access the given URL.
  * \param url The URL to evaluate
- * \param loadPendingPolicies Whether or not to load pending policy files while checking
  * \param allowedSandboxesRemote The sandboxes that are allowed to access remote URLs in this case.
  *                               Can be a bitwise expression of sandboxes.
  * \param allowedSandboxesLocal The sandboxes that are allowed to access local URLs in this case.
@@ -328,11 +296,9 @@ bool SecurityManager::evaluateSandbox(SANDBOXTYPE sandbox, int allowedSandboxes)
  * \see SecurityManager::evaluateSandboxURL()
  * \see SecurityManager::evaluatePortURL()
  * \see SecurityManager::evaluateLocalDirectoryURL()
- * \see SecurityManager::evaluatePoliciesURL()
  */
-SecurityManager::EVALUATIONRESULT SecurityManager::evaluateURL(const URLInfo& url,
-		bool loadPendingPolicies,	int allowedSandboxesRemote, int allowedSandboxesLocal,
-		bool restrictLocalDirectory)
+SecurityManager::EVALUATIONRESULT SecurityManager::evaluateURLStatic(const URLInfo& url,
+		int allowedSandboxesRemote, int allowedSandboxesLocal, bool restrictLocalDirectory)
 {
 	//Check the sandbox first
 	EVALUATIONRESULT sandboxResult =
@@ -353,11 +319,6 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluateURL(const URLInfo& ur
 			return restrictLocalDirResult;
 	}
 
-	//Finally, check policy files
-	EVALUATIONRESULT policiesResult = evaluatePoliciesURL(url, loadPendingPolicies);
-	if(policiesResult != ALLOWED)
-		return policiesResult;
-
 	//All checks passed, so we allow the URL connection
 	return ALLOWED;
 }
@@ -395,7 +356,7 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluateSandboxURL(const URLI
 SecurityManager::EVALUATIONRESULT SecurityManager::evaluateLocalDirectoryURL(const URLInfo& url)
 {
 	//The URL is local and points to a directory above the origin
-	if(url.getProtocol() == "file" && !url.isSubOf(sys->getOrigin()))
+	if(url.getProtocol() == "file" && !url.isSubOf(getSys()->getOrigin()))
 		return NA_RESTRICT_LOCAL_DIRECTORY;
 
 	return ALLOWED;
@@ -441,26 +402,32 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluatePoliciesURL(const URL
 		bool loadPendingPolicies)
 {
 	//This check doesn't apply to local files
-	if(url.getProtocol() == "file" && sys->getOrigin().getProtocol() == "file")
+	if(url.getProtocol() == "file" && getSys()->getOrigin().getProtocol() == "file")
 		return ALLOWED;
 
-	LOG(LOG_NO_INFO, _("SECURITY: Evaluating URL for cross domain policies:"));
-	LOG(LOG_NO_INFO, _("SECURITY: --> URL:    ") << url);
-	LOG(LOG_NO_INFO, _("SECURITY: --> Origin: ") << sys->getOrigin());
+	//Streaming from RTMP is always allowed (see
+	//http://forums.adobe.com/thread/422391)
+	if(url.getProtocol() == "rtmp" ||
+	   url.getProtocol() == "rtmpe" ||
+	   url.getProtocol() == "rtmps")
+		return ALLOWED;
+
+	LOG(LOG_INFO, _("SECURITY: Evaluating URL for cross domain policies:"));
+	LOG(LOG_INFO, _("SECURITY: --> URL:    ") << url);
+	LOG(LOG_INFO, _("SECURITY: --> Origin: ") << getSys()->getOrigin());
 
 	//The URL has exactly the same domain name as the origin, always allowed
-	if(url.getProtocol() == sys->getOrigin().getProtocol() &&
-			url.getHostname() == sys->getOrigin().getHostname())
+	if(url.getProtocol() == getSys()->getOrigin().getProtocol() &&
+			url.getHostname() == getSys()->getOrigin().getHostname())
 	{
-		LOG(LOG_NO_INFO, _("SECURITY: Same hostname as origin, allowing"));
+		LOG(LOG_INFO, _("SECURITY: Same hostname as origin, allowing"));
 		return ALLOWED;
 	}
 
 	//Search for the policy files to check
 	URLPFileList* files = searchURLPolicyFiles(url, loadPendingPolicies);
-	
-	sem_wait(&mutex);
-	//-- Lock acquired
+
+	RecMutex::Lock l(mutex);
 
 	//Check the policy files
 	if(files != NULL)
@@ -468,23 +435,18 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluatePoliciesURL(const URL
 		URLPFileListConstIt it = files->begin();
 		for(; it != files->end(); ++it)
 		{
-			if((*it)->allowsAccessFrom(sys->getOrigin(), url))
+			if((*it)->allowsAccessFrom(getSys()->getOrigin(), url))
 			{
-				LOG(LOG_NO_INFO, _("SECURITY: ALLOWED: A policy file explicitly allowed access"));
+				LOG(LOG_INFO, _("SECURITY: ALLOWED: A policy file explicitly allowed access"));
 				delete files;
-
-				//++ Release lock
-				sem_post(&mutex);
 				return ALLOWED;
 			}
 		}
 	}
 
-	LOG(LOG_NO_INFO, _("SECURITY: DISALLOWED: No policy file explicitly allowed access"));
+	LOG(LOG_INFO, _("SECURITY: DISALLOWED: No policy file explicitly allowed access"));
 	delete files;
 
-	//++ Release lock
-	sem_post(&mutex);
 	return NA_CROSSDOMAIN_POLICY;
 }
 
@@ -502,12 +464,12 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluateHeader(const URLInfo&
 		const tiny_string& header, bool loadPendingPolicies)
 {
 	//This check doesn't apply to local files
-	if(url.getProtocol() == "file" && sys->getOrigin().getProtocol() == "file")
+	if(url.getProtocol() == "file" && getSys()->getOrigin().getProtocol() == "file")
 		return ALLOWED;
 
-	LOG(LOG_NO_INFO, _("SECURITY: Evaluating header for cross domain policies ('") << header << "'):");
-	LOG(LOG_NO_INFO, _("SECURITY: --> URL: ") << url);
-	LOG(LOG_NO_INFO, _("SECURITY: --> Origin: ") << sys->getOrigin());
+	LOG(LOG_INFO, _("SECURITY: Evaluating header for cross domain policies ('") << header << "'):");
+	LOG(LOG_INFO, _("SECURITY: --> URL: ") << url);
+	LOG(LOG_INFO, _("SECURITY: --> Origin: ") << getSys()->getOrigin());
 
 	string headerStrLower(header.raw_buf());
 	transform(headerStrLower.begin(), headerStrLower.end(), headerStrLower.begin(), ::tolower);
@@ -532,23 +494,22 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluateHeader(const URLInfo&
 			headerStr == "uri" && headerStr == "user-agent" && headerStr == "vary" && headerStr == "via" &&
 			headerStr == "warning" && headerStr == "www-authenticate" && headerStr == "x-flash-version")
 	{
-		LOG(LOG_NO_INFO, _("SECURITY: DISALLOWED: Header is restricted"));
+		LOG(LOG_INFO, _("SECURITY: DISALLOWED: Header is restricted"));
 		return NA_HEADER;
 	}
 
 	//The URL has exactly the same domain name as the origin, always allowed
-	if(url.getProtocol() == sys->getOrigin().getProtocol() &&
-			url.getHostname() == sys->getOrigin().getHostname())
+	if(url.getProtocol() == getSys()->getOrigin().getProtocol() &&
+			url.getHostname() == getSys()->getOrigin().getHostname())
 	{
-		LOG(LOG_NO_INFO, _("SECURITY: ALLOWED: Same hostname as origin"));
+		LOG(LOG_INFO, _("SECURITY: ALLOWED: Same hostname as origin"));
 		return ALLOWED;
 	}
 
 	//Search for the policy files to check
 	URLPFileList* files = searchURLPolicyFiles(url, loadPendingPolicies);
-	
-	sem_wait(&mutex);
-	//-- Lock acquired
+
+	RecMutex::Lock l(mutex);
 
 	//Check the policy files
 	if(files != NULL)
@@ -556,23 +517,18 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluateHeader(const URLInfo&
 		URLPFileListConstIt it = files->begin();
 		for(; it != files->end(); ++it)
 		{
-			if((*it)->allowsHTTPRequestHeaderFrom(sys->getOrigin(), url, headerStrLower))
+			if((*it)->allowsHTTPRequestHeaderFrom(getSys()->getOrigin(), url, headerStrLower))
 			{
-				LOG(LOG_NO_INFO, _("SECURITY: ALLOWED: A policy file explicitly allowed the header"));
+				LOG(LOG_INFO, _("SECURITY: ALLOWED: A policy file explicitly allowed the header"));
 				delete files;
-
-				//++ Release lock
-				sem_post(&mutex);
 				return ALLOWED;
 			}
 		}
 	}
 
-	LOG(LOG_NO_INFO, _("SECURITY: DISALLOWED: No policy file explicitly allowed the header"));
+	LOG(LOG_INFO, _("SECURITY: DISALLOWED: No policy file explicitly allowed the header"));
 	delete files;
 
-	//++ Release lock
-	sem_post(&mutex);
 	return NA_CROSSDOMAIN_POLICY;
 }
 
@@ -587,8 +543,6 @@ PolicyFile::PolicyFile(URLInfo _url, TYPE _type):
 	url(_url),type(_type),valid(false),ignore(false),
 	loaded(false),siteControl(NULL)
 {
-	sem_init(&mutex,0,1);
-	//== Lock initialized
 }
 
 /**
@@ -600,15 +554,10 @@ PolicyFile::PolicyFile(URLInfo _url, TYPE _type):
  */
 PolicyFile::~PolicyFile()
 {
-	sem_wait(&mutex);
-	//-- Lock acquired
-	
-	for(list<PolicyAllowAccessFrom*>::iterator i = allowAccessFrom.begin(); 
+	Mutex::Lock l(mutex);
+	for(list<PolicyAllowAccessFrom*>::iterator i = allowAccessFrom.begin();
 			i != allowAccessFrom.end(); ++i)
 		delete (*i);
-
-	//== Destroy lock
-	sem_destroy(&mutex);
 }
 
 /**
@@ -641,15 +590,10 @@ URLPolicyFile::URLPolicyFile(const URLInfo& _url):
  */
 URLPolicyFile::~URLPolicyFile()
 {
-	sem_wait(&mutex);
-	//-- Lock acquired
-
+	Mutex::Lock l(mutex);
 	for(list<PolicyAllowHTTPRequestHeadersFrom*>::iterator i = allowHTTPRequestHeadersFrom.begin();
 			i != allowHTTPRequestHeadersFrom.end(); ++i)
 		delete (*i);
-
-	//++ Release lock
-	sem_post(&mutex);
 }
 
 /**
@@ -662,22 +606,15 @@ URLPolicyFile::~URLPolicyFile()
  */
 URLPolicyFile* URLPolicyFile::getMasterPolicyFile()
 {
-	sem_wait(&mutex);
-	//-- Lock acquired
+	Mutex::Lock l(mutex);
 
 	if(isMaster())
-	{
-		//++ Release lock
-		sem_post(&mutex);
 		return this;
-	}
 
-	URLPolicyFile* file = sys->securityManager->getURLPolicyFileByURL(url.goToURL("/crossdomain.xml"));
+	URLPolicyFile* file = getSys()->securityManager->getURLPolicyFileByURL(url.goToURL("/crossdomain.xml"));
 	if(file == NULL)
-		file = sys->securityManager->addURLPolicyFile(url.goToURL("/crossdomain.xml"));
+		file = getSys()->securityManager->addURLPolicyFile(url.goToURL("/crossdomain.xml"));
 
-	//++ Release lock
-	sem_post(&mutex);
 	return file;
 }
 
@@ -710,14 +647,12 @@ void URLPolicyFile::load()
 
 	URLPolicyFile* master = getMasterPolicyFile();
 
-	sem_wait(&mutex);
-	//-- Lock acquired
-
+	Mutex::Lock l(mutex);
 	//Check if this file is allowed/ignored by the master policy file
 	if(!isMaster())
 	{
 		//Load master policy file if not loaded yet
-		sys->securityManager->loadPolicyFile(master);
+		getSys()->securityManager->loadPolicyFile(master);
 		//Master policy file found and valid and has a site-control entry
 		if(master->isValid() && master->getSiteControl() != NULL)
 		{
@@ -733,7 +668,7 @@ void URLPolicyFile::load()
 	}
 
 	//No caching needed for this download, we don't expect very big files
-	Downloader* downloader=sys->downloadManager->download(url, false);
+	Downloader* downloader=getSys()->downloadManager->download(url, false, NULL);
 
 	//Wait until the file is fetched
 	downloader->waitForTermination();
@@ -746,11 +681,11 @@ void URLPolicyFile::load()
 		URLInfo newURL(downloader->getURL());
 		if(url.getHostname() != newURL.getHostname())
 		{
-			LOG(LOG_NO_INFO, _("SECURITY: Policy file was redirected to other domain, marking invalid"));
+			LOG(LOG_INFO, _("SECURITY: Policy file was redirected to other domain, marking invalid"));
 			valid = false;
 		}
 		url = newURL;
-		LOG(LOG_NO_INFO, _("SECURITY: Policy file was redirected"));
+		LOG(LOG_INFO, _("SECURITY: Policy file was redirected"));
 	}
 
 	//Policy files must have on of the following content-types to be valid:
@@ -761,7 +696,7 @@ void URLPolicyFile::load()
 			contentType != "application/xml" &&
 			contentType != "application/xhtml+xml")
 	{
-		LOG(LOG_NO_INFO, _("SECURITY: Policy file has an invalid content-type, marking invalid"));
+		LOG(LOG_INFO, _("SECURITY: Policy file has an invalid content-type, marking invalid"));
 		valid = false;
 	}
 
@@ -776,7 +711,7 @@ void URLPolicyFile::load()
 				master->getSiteControl()->getPermittedPolicies() == PolicySiteControl::BY_CONTENT_TYPE &&
 				contentType != "text/x-cross-domain-policy")
 		{
-			LOG(LOG_NO_INFO, _("SECURITY: Policy file content-type isn't strict, marking invalid"));
+			LOG(LOG_INFO, _("SECURITY: Policy file content-type isn't strict, marking invalid"));
 			ignore = true;
 		}
 	}
@@ -792,7 +727,7 @@ void URLPolicyFile::load()
 		s.read((char*)buf,bufLength);
 
 		//We're done with the downloader, lets destroy ASAP
-		sys->downloadManager->destroy(downloader);
+		getSys()->downloadManager->destroy(downloader);
 
 		CrossDomainPolicy::POLICYFILESUBTYPE parserSubtype = CrossDomainPolicy::NONE;
 		if(subtype == HTTP)
@@ -827,6 +762,8 @@ void URLPolicyFile::load()
 			elementType = parser.getNextElement();
 		}
 
+		delete[] buf;
+
 		//The last element was INVALID
 		if(elementType == CrossDomainPolicy::INVALID)
 			valid = false;
@@ -854,11 +791,8 @@ void URLPolicyFile::load()
 	{
 		//Failed to download the file, marking this file as invalid
 		valid = false;
-		sys->downloadManager->destroy(downloader);
+		getSys()->downloadManager->destroy(downloader);
 	}
-
-	//++ Release lock
-	sem_post(&mutex);
 }
 
 /**
