@@ -19,30 +19,59 @@
 
 #include "swf.h"
 #include "logger.h"
-#include "parsing/streams.h"
-#include "backends/netutils.h"
-#ifndef WIN32
-#include <sys/resource.h>
-#include <unistd.h>
+#include "backends/security.h"
+#include "platforms/engineutils.h"
+#ifndef _WIN32
+#	include <sys/resource.h>
 #endif
-#include <iostream>
-#include <fstream>
 #include "compat.h"
 
-#ifdef WIN32
-#include <windows.h>
-#endif
-#include <SDL.h>
-#ifdef WIN32
-#undef main
-#endif
 
 using namespace std;
 using namespace lightspark;
 
-TLSDATA DLL_PUBLIC SystemState* sys;
-TLSDATA DLL_PUBLIC RenderThread* rt=NULL;
-TLSDATA DLL_PUBLIC ParseThread* pt=NULL;
+class StandaloneEngineData: public EngineData
+{
+	guint destroyHandlerId;
+public:
+	StandaloneEngineData() : destroyHandlerId(0)
+	{
+#ifndef _WIN32
+		visual = XVisualIDFromVisual(gdk_x11_visual_get_xvisual(gdk_visual_get_system()));
+#endif
+	}
+	~StandaloneEngineData()
+	{
+		if(destroyHandlerId && widget)
+			g_signal_handler_disconnect(widget, destroyHandlerId);
+	}
+	static void StandaloneDestroy(GtkWidget *widget, gpointer data)
+	{
+		StandaloneEngineData* e = (StandaloneEngineData*)data;
+		RecMutex::Lock l(e->mutex);
+		/* no need to destroy it - it's already done */
+		e->widget = NULL;
+		getSys()->setShutdownFlag();
+	}
+	GtkWidget* createGtkWidget()
+	{
+		GtkWidget* window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
+		gtk_window_set_title((GtkWindow*)window,"Lightspark");
+		destroyHandlerId = g_signal_connect(window,"destroy",G_CALLBACK(StandaloneDestroy),this);
+		return window;
+	}
+	NativeWindow getWindowForGnash()
+	{
+		/* passing and invalid window id to gnash makes
+		 * it create its own window */
+		return 0;
+	}
+	void stopMainDownload() {}
+	bool isSizable() const
+	{
+		return true;
+	}
+};
 
 int main(int argc, char* argv[])
 {
@@ -52,16 +81,29 @@ int main(int argc, char* argv[])
 #ifdef PROFILING_SUPPORT
 	char* profilingFileName=NULL;
 #endif
-	SecurityManager::SANDBOXTYPE sandboxType=SecurityManager::REMOTE;
+	char *HTTPcookie=NULL;
+	SecurityManager::SANDBOXTYPE sandboxType=SecurityManager::LOCAL_WITH_FILE;
 	bool useInterpreter=true;
 	bool useJit=false;
-	LOG_LEVEL log_level=LOG_NOT_IMPLEMENTED;
+	bool exitOnError=false;
+	LOG_LEVEL log_level=LOG_INFO;
 
 	setlocale(LC_ALL, "");
-	bindtextdomain("lightspark", "/usr/share/locale");
+#ifdef _WIN32
+	const char* localedir = getExectuablePath();
+#else
+	const char* localedir = "/usr/share/locale";
+#endif
+	bindtextdomain("lightspark", localedir);
 	textdomain("lightspark");
 
-	cout << "Lightspark version " << VERSION << " Copyright 2009-2011 Alessandro Pignotti and others" << endl;
+	LOG(LOG_INFO,"Lightspark version " << VERSION << " Copyright 2009-2011 Alessandro Pignotti and others");
+
+	//Make GTK thread enabled
+	g_thread_init(NULL);
+	gdk_threads_init();
+	//Give GTK a chance to parse its own options
+	gtk_init (&argc, &argv);
 
 	for(int i=1;i<argc;i++)
 	{
@@ -90,7 +132,7 @@ int main(int argc, char* argv[])
 				break;
 			}
 
-			log_level=(LOG_LEVEL)atoi(argv[i]);
+			log_level=(LOG_LEVEL) min(4, max(0, atoi(argv[i])));
 		}
 		else if(strcmp(argv[i],"-p")==0 || 
 			strcmp(argv[i],"--parameters-file")==0)
@@ -139,6 +181,20 @@ int main(int argc, char* argv[])
 		{
 			exit(0);
 		}
+		else if(strcmp(argv[i],"--exit-on-error")==0)
+		{
+			exitOnError = true;
+		}
+		else if(strcmp(argv[i],"--HTTP-cookies")==0)
+		{
+			i++;
+			if(i==argc)
+			{
+				fileName=NULL;
+				break;
+			}
+			HTTPcookie=argv[i];
+		}
 		else
 		{
 			//No options flag, so set the swf file name
@@ -153,17 +209,18 @@ int main(int argc, char* argv[])
 
 	if(fileName==NULL)
 	{
-		cout << endl << "Usage: " << argv[0] << " [--url|-u http://loader.url/file.swf]" << 
-			" [--disable-interpreter|-ni] [--enable-jit|-j] [--log-level|-l 0-4]" << 
+		LOG(LOG_ERROR, endl << "Usage: " << argv[0] << " [--url|-u http://loader.url/file.swf]" <<
+			" [--disable-interpreter|-ni] [--enable-jit|-j] [--log-level|-l 0-4]" <<
 			" [--parameters-file|-p params-file] [--security-sandbox|-s sandbox]" <<
+			" [--exit-on-error] [--HTTP-cookies cookie]" <<
 #ifdef PROFILING_SUPPORT
-			" [--profiling-output|-o profiling-file]" << 
+			" [--profiling-output|-o profiling-file]" <<
 #endif
-			" <file.swf>" << endl;
+			" <file.swf>");
 		exit(1);
 	}
 
-#ifndef WIN32
+#ifndef _WIN32
 	struct rlimit rl;
 	getrlimit(RLIMIT_AS,&rl);
 	rl.rlim_cur=400000000;
@@ -172,36 +229,39 @@ int main(int argc, char* argv[])
 
 #endif
 
-	Log::initLogging(log_level);
-	ifstream f(fileName);
+	Log::setLogLevel(log_level);
+	ifstream f(fileName, ios::in|ios::binary);
 	f.seekg(0, ios::end);
 	uint32_t fileSize=f.tellg();
 	f.seekg(0, ios::beg);
 	if(!f)
 	{
-		cout << argv[0] << ": " << fileName << ": No such file or directory" << endl;
+		LOG(LOG_ERROR, argv[0] << ": " << fileName << ": No such file or directory");
 		exit(2);
 	}
 	f.exceptions ( istream::eofbit | istream::failbit | istream::badbit );
 	cout.exceptions( ios::failbit | ios::badbit);
 	cerr.exceptions( ios::failbit | ios::badbit);
-	ParseThread* pt = new ParseThread(NULL,f);
+	ParseThread* pt = new ParseThread(f);
 	SystemState::staticInit();
 	//NOTE: see SystemState declaration
-	sys=new SystemState(pt, fileSize);
+	SystemState* sys =new SystemState(pt, fileSize);
+	setTLSSys(sys);
+	sys->setDownloadedPath(fileName);
 
 	//This setting allows qualifying filename-only paths to fully qualified paths
 	//When the URL parameter is set, set the root URL to the given parameter
 	if(url)
 	{
 		sys->setOrigin(url, fileName);
+		sandboxType = SecurityManager::REMOTE;
 	}
-#ifndef WIN32
+#ifndef _WIN32
 	//When running in a local sandbox, set the root URL to the current working dir
 	else if(sandboxType != SecurityManager::REMOTE)
 	{
 		char * cwd = get_current_dir_name();
-		string cwdStr = string("file://") + string(cwd, true);
+		string cwdStr = string("file://") + string(cwd);
 		free(cwd);
 		cwdStr += "/";
 		sys->setOrigin(cwdStr, fileName);
@@ -210,7 +270,7 @@ int main(int argc, char* argv[])
 	else
 	{
 		sys->setOrigin(string("file://") + fileName);
-		LOG(LOG_NO_INFO, _("Warning: running with no origin URL set."));
+		LOG(LOG_INFO, _("Warning: running with no origin URL set."));
 	}
 
 	//One of useInterpreter or useJit must be enabled
@@ -221,36 +281,41 @@ int main(int argc, char* argv[])
 	}
 	sys->useInterpreter=useInterpreter;
 	sys->useJit=useJit;
+	sys->exitOnError=exitOnError;
 	if(paramsFileName)
 		sys->parseParametersFromFile(paramsFileName);
 #ifdef PROFILING_SUPPORT
 	if(profilingFileName)
 		sys->setProfilingOutput(profilingFileName);
 #endif
-	
-	SDL_Init ( SDL_INIT_VIDEO |SDL_INIT_EVENTTHREAD );
-	sys->setParamsAndEngine(SDL, NULL);
+	if(HTTPcookie)
+		sys->setCookies(HTTPcookie);
+
+	sys->setParamsAndEngine(new StandaloneEngineData(), true);
+
 	sys->securityManager->setSandboxType(sandboxType);
 	if(sandboxType == SecurityManager::REMOTE)
-		LOG(LOG_NO_INFO, _("Running in remote sandbox"));
+		LOG(LOG_INFO, _("Running in remote sandbox"));
 	else if(sandboxType == SecurityManager::LOCAL_WITH_NETWORK)
-		LOG(LOG_NO_INFO, _("Running in local-with-networking sandbox"));
+		LOG(LOG_INFO, _("Running in local-with-networking sandbox"));
 	else if(sandboxType == SecurityManager::LOCAL_WITH_FILE)
-		LOG(LOG_NO_INFO, _("Running in local-with-filesystem sandbox"));
+		LOG(LOG_INFO, _("Running in local-with-filesystem sandbox"));
 	else if(sandboxType == SecurityManager::LOCAL_TRUSTED)
-		LOG(LOG_NO_INFO, _("Running in local-trusted sandbox"));
+		LOG(LOG_INFO, _("Running in local-trusted sandbox"));
 
 	sys->downloadManager=new StandaloneDownloadManager();
 
 	//Start the parser
 	sys->addJob(pt);
 
-	sys->wait();
-	delete sys;
+	gdk_threads_enter();
+	gtk_main();
+	gdk_threads_leave();
+
+	sys->destroy();
 	delete pt;
 
 	SystemState::staticDeinit();
-	SDL_Quit();
 	return 0;
 }
 
