@@ -24,6 +24,7 @@
 #include "npscriptobject.h"
 
 using namespace std;
+using namespace lightspark;
 
 /* -- NPIdentifierObject -- */
 // Constructors
@@ -42,12 +43,12 @@ NPIdentifierObject::NPIdentifierObject(int32_t value)
 NPIdentifierObject::NPIdentifierObject(const ExtIdentifier& value)
 {
 	// It is possible we got a down-casted ExtIdentifier, so lets check for that
-	try
+	const NPIdentifierObject* npio = dynamic_cast<const NPIdentifierObject*>(&value);
+	if(npio)
 	{
-		dynamic_cast<const NPIdentifierObject&>(value).copy(identifier);
+		npio->copy(identifier);
 	}
-	// We got a real ExtIdentifier, lets convert it
-	catch(std::bad_cast&)
+	else
 	{
 		if(value.getType() == EI_STRING)
 			identifier = NPN_GetStringIdentifier(value.getString().c_str());
@@ -95,16 +96,11 @@ void NPIdentifierObject::copy(NPIdentifier& dest) const
 // Comparator
 bool NPIdentifierObject::operator<(const ExtIdentifier& other) const
 {
-	// It is possible we got a down-casted NPIdentifierObject, so lets check for that
-	try
-	{
-		return identifier < dynamic_cast<const NPIdentifierObject&>(other).getNPIdentifier();
-	}
-	// We got a real ExtIdentifier, let ExtIdentifier::operator< handle this
-	catch(std::bad_cast&)
-	{
+	const NPIdentifierObject* npi = dynamic_cast<const NPIdentifierObject*>(&other);
+	if(npi)
+		return identifier < npi->getNPIdentifier();
+	else
 		return ExtIdentifier::operator<(other);
-	}
 }
 
 // Type determination
@@ -289,16 +285,11 @@ NPObject* NPObjectObject::getNPObject(NPP instance, const lightspark::ExtObject&
 		{
 			property = obj.getProperty(i);
 
-			// The returned property might be an NPVariantObject, which would save us a conversion
-			try
-			{
-				dynamic_cast<NPVariantObject&>(*property).copy(varProperty);
-			}
-			// We got a real ExtVariant, so we need to convert it to NPVariantObject first
-			catch(std::bad_cast&)
-			{
+			NPVariantObject* npv = dynamic_cast<NPVariantObject*>(property);
+			if(npv)
+				npv->copy(varProperty);
+			else
 				NPVariantObject(instance, *property).copy(varProperty);
-			}
 
 			// Push the value onto the newly created Array
 			NPN_Invoke(instance, result, NPN_GetStringIdentifier("push"), &varProperty, 1, &resultVariant);
@@ -322,17 +313,11 @@ NPObject* NPObjectObject::getNPObject(NPP instance, const lightspark::ExtObject&
 			for(uint32_t i = 0; i < count; i++)
 			{
 				property = obj.getProperty(*ids[i]);
-
-				// The returned property might be an NPVariantObject, which would save us a conversion
-				try
-				{
-					dynamic_cast<NPVariantObject&>(*property).copy(varProperty);
-				}
-				// We got a real ExtVariant, so we need to convert it to NPVariantObject first
-				catch(std::bad_cast&)
-				{
+				NPVariantObject* npv = dynamic_cast<NPVariantObject*>(property);
+				if(npv)
+					npv->copy(varProperty);
+				else
 					NPVariantObject(instance, *property).copy(varProperty);
-				}
 
 				// Set the properties
 				NPN_SetProperty(instance, result, NPIdentifierObject(*ids[i]).getNPIdentifier(), &varProperty);
@@ -386,12 +371,12 @@ NPVariantObject::NPVariantObject(NPP _instance, bool value) : instance(_instance
 NPVariantObject::NPVariantObject(NPP _instance, const ExtVariant& value) : instance(_instance)
 {
 	// It's possible we got a down-casted NPVariantObject, so lets check for it
-	try
+	const NPVariantObject* npv = dynamic_cast<const NPVariantObject*>(&value);
+	if(npv)
 	{
-		dynamic_cast<const NPVariantObject&>(value).copy(variant);
+		npv->copy(variant);
 	}
-	// Seems we got a real ExtVariant, lets convert
-	catch(std::bad_cast&)
+	else
 	{
 		switch(value.getType())
 		{
@@ -550,11 +535,7 @@ NPScriptObject::NPScriptObject(NPScriptObjectGW* _gw) :
 {
 	// This object is always created in the main plugin thread, so lets save
 	// so that we can check if we are in the main plugin thread later on.
-	mainThread = pthread_self();
-
-	// Only one external call can be made at a time
-	sem_init(&mutex, 0, 1);
-	sem_init(&externalCallsFinished, 0, 1);
+	mainThread = Thread::self();
 
 	setProperty("$version", "LNX 10,2,"SHORTVERSION);
 
@@ -577,23 +558,21 @@ NPScriptObject::NPScriptObject(NPScriptObjectGW* _gw) :
 // Destructor preparator
 void NPScriptObject::destroy()
 {
-	sem_wait(&mutex);
+	mutex.lock();
 	// Prevents new external calls from continuing
 	shuttingDown = true;
-	sem_post(&mutex);
+	mutex.unlock();
 
 	// If an external call is running, wake it up
 	if(callStatusses.size() > 0)
-		sem_post(callStatusses.top());
+		callStatusses.top()->signal();
 	// Wait for all external calls to finish
-	sem_wait(&externalCallsFinished);
+	Mutex::Lock l(externalCall);
 }
 
 // Destructor
 NPScriptObject::~NPScriptObject()
 {
-	sem_destroy(&mutex);
-
 	std::map<NPIdentifierObject, lightspark::ExtCallback*>::iterator meth_it = methods.begin();
 	while(meth_it != methods.end())
 	{
@@ -616,8 +595,10 @@ bool NPScriptObject::invoke(NPIdentifier id, const NPVariant* args, uint32_t arg
 	if(it == methods.end())
 		return false;
 
+	LOG(LOG_CALLS,"Plugin callback from the browser: " << objId.getString());
+
 	// Convert raw arguments to objects
-	const lightspark::ExtVariant* objArgs[argc];
+	const lightspark::ExtVariant** objArgs = g_newa(const lightspark::ExtVariant*,argc);
 	for(uint32_t i = 0; i < argc; i++)
 		objArgs[i] = new NPVariantObject(instance, args[i]);
 
@@ -748,36 +729,51 @@ bool NPScriptObject::enumerate(lightspark::ExtIdentifier*** ids, uint32_t* count
 
 // ExtScriptObject interface: calling external methods
 bool NPScriptObject::callExternal(const lightspark::ExtIdentifier& id,
-		const lightspark::ExtVariant** args, uint32_t argc, lightspark::ExtVariant** result)
+		const lightspark::ExtVariant** args, uint32_t argc, lightspark::ASObject** result)
 {
 	// Make sure we are the only external call being executed
-	sem_wait(&mutex);
-	
+	mutex.lock();
 	// If we are shutting down, then don't even continue
 	if(shuttingDown)
 	{
-		sem_post(&mutex);
+		mutex.unlock();
 		return false;
 	}
 
 	// If we are the first external call, then indicate that an external call is running
 	if(callStatusses.size() == 0)
-		sem_wait(&externalCallsFinished);
+		externalCall.lock();
 
 	// Signals when the async has been completed
 	// True if NPN_Invoke succeeded
 	bool success = false;
 
 	// Used to signal completion of asynchronous external call
-	sem_t callStatus;
-	sem_init(&callStatus, 0, 0);
-	// Add this callStatus semaphore to the list of running call statusses to be cleaned up on shutdown
+	Semaphore callStatus(0);
+	// Add this callStatus semaphore to the list of running call statuses to be cleaned up on shutdown
 	callStatusses.push(&callStatus);
-	std::string scriptString = "(" + id.getString() + ")";
+	//We forge an anonymous function with the right number of arguments
+	std::string argsString;
+	for(uint32_t i=0;i<argc;i++)
+	{
+		char buf[20];
+		if((i+1)==argc)
+			snprintf(buf,20,"a%u",i);
+		else
+			snprintf(buf,20,"a%u,",i);
+		argsString += buf;
+	}
+
+	std::string scriptString = "(function(";
+	scriptString += argsString;
+	scriptString += ") { return (" + id.getString();
+	scriptString += ")(" + argsString + "); })";
+
+	LOG(LOG_CALLS,"Invoking " << scriptString << " in the browser ");
+
 	EXT_CALL_DATA data = {
-		&mainThread,
+		mainThread,
 		instance,
-		NPIdentifierObject(id).getNPIdentifier(),
 		scriptString.c_str(),
 		args,
 		argc,
@@ -787,11 +783,11 @@ bool NPScriptObject::callExternal(const lightspark::ExtIdentifier& id,
 	};
 
 	// Called JS may invoke a callback, which in turn may invoke another external method, which needs this mutex
-	sem_post(&mutex);
+	mutex.unlock();
 
 	// We are in the main thread,
 	// so we can call the method ourselves synchronously straight away
-	if(pthread_equal(pthread_self(), mainThread))
+	if(Thread::self() == mainThread)
 		callExternal(&data);
 	// We are not in the main thread
 	else
@@ -799,7 +795,7 @@ bool NPScriptObject::callExternal(const lightspark::ExtIdentifier& id,
 		// Main thread is not occupied by an invoked callback,
 		// so ask the browser to asynchronously call our external function.
 		if(currentCallback == NULL)
-			NPN_PluginThreadAsyncCall(instance, &callExternal, &data);
+			NPN_PluginThreadAsyncCall(instance, &NPScriptObject::callExternal, &data);
 		// Main thread is occupied by an invoked callback.
 		// Wake it up and ask it run our external call
 		else
@@ -810,94 +806,84 @@ bool NPScriptObject::callExternal(const lightspark::ExtIdentifier& id,
 	}
 
 	// Wait for the (possibly asynchronously) called function to finish
-	sem_wait(&callStatus);
+	callStatus.wait();
 
-	sem_wait(&mutex);
+	mutex.lock();
 
 	// This call status doesn't need to be cleaned up anymore on shutdown
 	callStatusses.pop();
-	sem_destroy(&callStatus);
 
 	// If we are the last external call, then indicate that all external calls are now finished
 	if(callStatusses.size() == 0)
-		sem_post(&externalCallsFinished);
+		externalCall.unlock();
 
-	sem_post(&mutex);
+	mutex.unlock();
 	return success;
 }
+
 void NPScriptObject::callExternal(void* d)
 {
 	EXT_CALL_DATA* data = static_cast<EXT_CALL_DATA*>(d);
+	nsPluginInstance* plugin = (nsPluginInstance*)data->instance->pdata;
+	lightspark::SystemState* prevSys = getSys();
+	setTLSSys(plugin->m_sys);
+
 
 	// Assert we are in the main plugin thread
-	assert(pthread_equal(pthread_self(), *data->mainThread));
+	assert(Thread::self() == data->mainThread);
 
 	// This will hold the result from our NPN_Invoke(Default) call
 	NPVariant resultVariant;
 
-	// These will get passed as arguments to NPN_Invoke(Default)
-	NPVariant variantArgs[data->argc];
-	for(uint32_t i = 0; i < data->argc; i++)
-		NPVariantObject(data->instance, *(data->args[i])).copy(variantArgs[i]);
-
 	NPObject* windowObject;
 	NPN_GetValue(data->instance, NPNVWindowNPObject, &windowObject);
 
-	// Lets try to invoke the identifier as if it is a function name
-	*(data->success) = NPN_Invoke(data->instance, windowObject, data->id,
-				variantArgs, data->argc, &resultVariant);
-	if(!*(data->success))
-	{
-		// Simple invoke failed, now try to evaluate the "identifier" as a script
-		NPString script;
-		script.UTF8Characters = data->scriptString;
-		script.UTF8Length = strlen(data->scriptString);
-		*(data->success) = NPN_Evaluate(data->instance, windowObject, &script, &resultVariant);
+	NPString script;
+	script.UTF8Characters = data->scriptString;
+	script.UTF8Length = strlen(data->scriptString);
+	*(data->success) = NPN_Evaluate(data->instance, windowObject, &script, &resultVariant);
 
-		if(*(data->success) && NPVARIANT_IS_OBJECT(resultVariant))
+	//Evaluate should have returned a function object, if not bail out
+	if(*(data->success))
+	{
+		//SUCCESS
+		if(!NPVARIANT_IS_OBJECT(resultVariant))
 		{
-			// Evaluate didn't fail AND returned an object, lets try to invoke this object.
-			// An edge case which we can't handle is when the evaluation itself ran a function
-			// which returned another function.
-			// In this case both the first function and the second function will be called.
-			// Example:
-			// Identifier:          test()
-			// Definition of test:  function test() { alert('I meant to display this.'); return test2; }
-			// Definition of test2: function test2() { alert('I didnt mean to display this.'); }
-			// Running will result in both alerts popping up, while only  the first was meant to pop up.
+			//Something very wrong happended, our forged function is not a function!
+			LOG(LOG_ERROR,"Could not evaluate wrapper function in external interface");
+		}
+		else
+		{
+			// These will get passed as arguments to NPN_Invoke(Default)
+			NPVariant* variantArgs = g_newa(NPVariant,data->argc);
+			for(uint32_t i = 0; i < data->argc; i++)
+				NPVariantObject(data->instance, *(data->args[i])).copy(variantArgs[i]);
+
 			NPVariant evalResult = resultVariant;
 			NPObject* evalObj = NPVARIANT_TO_OBJECT(resultVariant);
-			bool evalSuccess = *(data->success);
 			// Lets try to invoke the default function on the object returned by the evaluation
 			*(data->success) = NPN_InvokeDefault(data->instance, evalObj,
 						variantArgs, data->argc, &resultVariant);
-			if(!*(data->success))
+
+			//Release the result fo the evaluation
+			NPN_ReleaseVariantValue(&evalResult);
+
+			// Release the converted arguments
+			for(uint32_t i = 0; i < data->argc; i++)
+				NPN_ReleaseVariantValue(&(variantArgs[i]));
+
+			if(*(data->success))
 			{
-				// Invocation of default function failed, it seems the evaluation didn't return a function.
-				// Restore previous results.
-				resultVariant = evalResult;
-				*(data->success) = evalSuccess;
-			}
-			else
-			{
-				// Invocation of the default function succeeded, lets release the intermediate object.
-				NPN_ReleaseVariantValue(&evalResult);
+				NPVariantObject tmp(data->instance, resultVariant);
+				*(data->result) = tmp.getASObject();
+				NPN_ReleaseVariantValue(&resultVariant);
 			}
+
 		}
 	}
 
-	// If NPN_Invoke(Default) succeeded, convert & copy its result and release it.
-	if(*(data->success))
-	{
-		*(data->result) = new NPVariantObject(data->instance, resultVariant);
-		NPN_ReleaseVariantValue(&resultVariant);
-	}
-
-	// Release the converted arguments
-	for(uint32_t i = 0; i < data->argc; i++)
-		NPN_ReleaseVariantValue(&(variantArgs[i]));
-
-	sem_post(data->callStatus);
+	data->callStatus->signal();
+	setTLSSys(prevSys);
 }
 
 void NPScriptObject::setException(const std::string& message) const
@@ -1086,28 +1072,28 @@ NPScriptObjectGW::~NPScriptObjectGW()
 // Properties
 bool NPScriptObjectGW::getProperty(NPObject* obj, NPIdentifier id, NPVariant* result)
 {
-	lightspark::SystemState* prevSys = sys;
-	sys = ((NPScriptObjectGW*) obj)->m_sys;
-	
+	lightspark::SystemState* prevSys = getSys();
+	setTLSSys( ((NPScriptObjectGW*) obj)->m_sys );
+
 	NPVariantObject* resultObj = ((NPScriptObjectGW*) obj)->so->getProperty(NPIdentifierObject(id));
 	if(resultObj == NULL)
 	{
-		sys = NULL;
+		setTLSSys( NULL );
 		return false;
 	}
 
 	resultObj->copy(*result);
 	delete resultObj;
 
-	sys = prevSys;
+	setTLSSys( prevSys );
 	return true;
 }
 
 // Enumeration
 bool NPScriptObjectGW::enumerate(NPObject* obj, NPIdentifier** value, uint32_t* count)
 {
-	lightspark::SystemState* prevSys = sys;
-	sys = ((NPScriptObjectGW*) obj)->m_sys;
+	lightspark::SystemState* prevSys = getSys();
+	setTLSSys( ((NPScriptObjectGW*) obj)->m_sys );
 
 	NPScriptObject* o = ((NPScriptObjectGW*) obj)->so;
 	lightspark::ExtIdentifier** ids = NULL;
@@ -1125,6 +1111,6 @@ bool NPScriptObjectGW::enumerate(NPObject* obj, NPIdentifier** value, uint32_t*
 	if(ids != NULL)
 		delete ids;
 
-	sys = prevSys;
+	setTLSSys( prevSys );
 	return success;
 }
