@@ -36,10 +36,7 @@
 #define PLUGIN_DESCRIPTION "Shockwave Flash 10.2 r"SHORTVERSION
 
 using namespace std;
-
-TLSDATA DLL_PUBLIC lightspark::SystemState* sys=NULL;
-TLSDATA DLL_PUBLIC lightspark::RenderThread* rt=NULL;
-TLSDATA DLL_PUBLIC lightspark::ParseThread* pt=NULL;
+using namespace lightspark;
 
 /**
  * \brief Constructor for NPDownloadManager
@@ -53,14 +50,6 @@ NPDownloadManager::NPDownloadManager(NPP _instance):instance(_instance)
 	type = NPAPI;
 }
 
-/**
- * \brief Destructor for NPDownloaderManager
- */
-NPDownloadManager::~NPDownloadManager()
-{
-	cleanUp();
-}
-
 /**
  * \brief Create a Downloader for an URL.
  *
@@ -72,6 +61,14 @@ NPDownloadManager::~NPDownloadManager()
  */
 lightspark::Downloader* NPDownloadManager::download(const lightspark::URLInfo& url, bool cached, lightspark::ILoadable* owner)
 {
+	// Handle RTMP requests internally, not through NPAPI
+	if(url.getProtocol()=="rtmp" ||
+	   url.getProtocol()=="rtmpe" ||
+	   url.getProtocol()=="rtmps")
+	{
+		return StandaloneDownloadManager::download(url, cached, owner);
+	}
+
 	LOG(LOG_INFO, _("NET: PLUGIN: DownloadManager::download '") << url.getParsedURL() << 
 			"'" << (cached ? _(" - cached") : ""));
 	//Register this download
@@ -91,6 +88,14 @@ lightspark::Downloader* NPDownloadManager::download(const lightspark::URLInfo& u
 lightspark::Downloader* NPDownloadManager::downloadWithData(const lightspark::URLInfo& url, const std::vector<uint8_t>& data, 
 		lightspark::ILoadable* owner)
 {
+	// Handle RTMP requests internally, not through NPAPI
+	if(url.getProtocol()=="rtmp" ||
+	   url.getProtocol()=="rtmpe" ||
+	   url.getProtocol()=="rtmps")
+	{
+		return StandaloneDownloadManager::downloadWithData(url, data, owner);
+	}
+
 	LOG(LOG_INFO, _("NET: PLUGIN: DownloadManager::downloadWithData '") << url.getParsedURL());
 	//Register this download
 	NPDownloader* downloader=new NPDownloader(url.getParsedURL(), data, instance, owner);
@@ -100,8 +105,13 @@ lightspark::Downloader* NPDownloadManager::downloadWithData(const lightspark::UR
 
 void NPDownloadManager::destroy(lightspark::Downloader* downloader)
 {
-	//Convert to a dynamic_cast if any other downloader is ever created by NPDownloadManager
-	NPDownloader* d=static_cast<NPDownloader*>(downloader);
+	NPDownloader* d=dynamic_cast<NPDownloader*>(downloader);
+	if(!d)
+	{
+		StandaloneDownloadManager::destroy(downloader);
+		return;
+	}
+
 	/*If the NP stream is already destroyed we can surely destroy the Downloader.
 	  Moreover, if ASYNC_DESTROY is already set, destroy is being called for the second time.
 	  This may happen when:
@@ -184,23 +194,59 @@ char* NPP_GetMIMEDescription(void)
 	return (char*)(MIME_TYPES_DESCRIPTION);
 }
 
+#ifdef _WIN32
+static void gtk_main_runner()
+{
+	gdk_threads_enter();
+	gtk_main();
+	gdk_threads_leave();
+}
+#endif
+
 /////////////////////////////////////
 // general initialization and shutdown
 //
+static Thread* gtkThread = NULL;
 NPError NS_PluginInitialize()
 {
-	LOG_LEVEL log_level = LOG_INFO;
+	LOG_LEVEL log_level = LOG_NOT_IMPLEMENTED;
+
+	/* setup glib/gtk, this is already done on firefox/linux, but needs to be done
+	 * on firefox/windows */
+	g_thread_init(NULL);
+	gdk_threads_init();
+	gtk_init(NULL, NULL);
+
 	char *envvar = getenv("LIGHTSPARK_PLUGIN_LOGLEVEL");
 	if (envvar)
 		log_level=(LOG_LEVEL) min(4, max(0, atoi(envvar)));
-	Log::initLogging(log_level);
+
+	envvar = getenv("LIGHTSPARK_PLUGIN_LOGFILE");
+	if (envvar)
+		Log::redirect(envvar);
+
+	Log::setLogLevel(log_level);
 	lightspark::SystemState::staticInit();
+#ifdef _WIN32
+	/* On win32, we link statically to gtk. Therefore we need to run
+	 * our own gtk main loop.
+	 * On linux, this is done by the firefox process.
+	 */
+	gtkThread = Thread::create(sigc::ptr_fun(&gtk_main_runner), true);
+#endif
 	return NPERR_NO_ERROR;
 }
 
 void NS_PluginShutdown()
 {
 	LOG(LOG_INFO,"Lightspark plugin shutdown");
+	if(gtkThread)
+	{
+		gdk_threads_enter();
+		gtk_main_quit();
+		gdk_threads_leave();
+		gtkThread->join();
+	}
 	lightspark::SystemState::staticDeinit();
 }
 
@@ -246,7 +292,7 @@ void NS_DestroyPluginInstance(nsPluginInstanceBase * aPlugin)
 {
 	if(aPlugin)
 		delete (nsPluginInstance *)aPlugin;
-	sys=NULL;
+	setTLSSys( NULL );
 }
 
 ////////////////////////////////////////
@@ -254,14 +300,12 @@ void NS_DestroyPluginInstance(nsPluginInstanceBase * aPlugin)
 // nsPluginInstance class implementation
 //
 nsPluginInstance::nsPluginInstance(NPP aInstance, int16_t argc, char** argn, char** argv) : 
-	nsPluginInstanceBase(), mInstance(aInstance),mInitialized(FALSE),mContainer(NULL),mWindow(0),
+	nsPluginInstanceBase(), mInstance(aInstance),mInitialized(FALSE),mWindow(0),
 	mainDownloaderStream(NULL),mainDownloader(NULL),scriptObject(NULL),m_pt(NULL)
 {
-	cout << "Lightspark version " << VERSION << " Copyright 2009-2011 Alessandro Pignotti and others" << endl;
-	sys=NULL;
+	LOG(LOG_INFO, "Lightspark version " << VERSION << " Copyright 2009-2011 Alessandro Pignotti and others");
+	setTLSSys( NULL );
 	m_sys=new lightspark::SystemState(NULL,0);
-	//As this is the plugin, enable fallback on Gnash for older clips
-	m_sys->enableGnashFallback();
 	//Files running in the plugin have REMOTE sandbox
 	m_sys->securityManager->setSandboxType(lightspark::SecurityManager::REMOTE);
 	//Find flashvars argument
@@ -289,13 +333,13 @@ nsPluginInstance::nsPluginInstance(NPP aInstance, int16_t argc, char** argn, cha
 		LOG(LOG_ERROR, "PLUGIN: Browser doesn't support NPRuntime");
 
 	//The sys var should be NULL in this thread
-	sys=NULL;
+	setTLSSys( NULL );
 }
 
 nsPluginInstance::~nsPluginInstance()
 {
 	//Shutdown the system
-	sys=m_sys;
+	setTLSSys(m_sys);
 	if(mainDownloader)
 		mainDownloader->stop();
 
@@ -306,14 +350,9 @@ nsPluginInstance::~nsPluginInstance()
 		m_pt->stop();
 	m_sys->setShutdownFlag();
 
-	m_sys->wait();
-
-	// Delete our external script object
-	delete m_sys->extScriptObject;
-
 	m_sys->destroy();
 	delete m_pt;
-	sys=NULL;
+	setTLSSys(NULL);
 }
 
 void nsPluginInstance::draw()
@@ -372,6 +411,54 @@ NPError nsPluginInstance::GetValue(NPPVariable aVariable, void *aValue)
 
 }
 
+#ifdef _WIN32
+/*
+ * Create a GtkWidget and embed it into that mWindow. This unifies the other code,
+ * because on any platform and standalone/plugin, we always handle GtkWidgets.
+ * Additionally, firefox always crashed on me when trying to directly draw into
+ * mWindow.
+ * This must be run in the gtk_main() thread for AttachThreadInput to make sense.
+ */
+GtkWidget* PluginEngineData::createGtkWidget()
+{
+	HWND parent_hwnd = instance->mWindow;
+
+	GtkWidget* widget=gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	/* Remove window decorations */
+	gtk_window_set_decorated((GtkWindow*)widget, FALSE);
+	/* Realize to construct hwnd */
+	gtk_widget_realize(widget);
+	HWND window = (HWND)GDK_WINDOW_HWND(gtk_widget_get_window(widget));
+	/* Set WS_CHILD, remove WS_POPUP - see MSDN on SetParent */
+	DWORD dwStyle = GetWindowLong (window, GWL_STYLE);
+	dwStyle |= WS_CHILD;
+	dwStyle &= ~WS_POPUP;
+	SetWindowLong(window, GWL_STYLE, dwStyle);
+	SetForegroundWindow(window);
+	/* Re-parent */
+	SetParent(window, parent_hwnd);
+	/* Attach our thread to that of the parent.
+	 * This ensures that we get messages for input events*/
+	DWORD parentThreadId;
+	if(!(parentThreadId = GetWindowThreadProcessId(parent_hwnd, NULL)))
+		LOG(LOG_ERROR,"GetWindowThreadProcessId failed");
+	DWORD myThreadId;
+	if(!(myThreadId = GetCurrentThreadId()))
+		LOG(LOG_ERROR,"GetCurrentThreadId failed");
+	if(!AttachThreadInput(myThreadId, parentThreadId, TRUE))
+		LOG(LOG_ERROR,"AttachThreadInput failed");
+	/* Set window size */
+	gtk_widget_set_size_request(widget, width, height);
+
+	return widget;
+}
+#else
+GtkWidget* PluginEngineData::createGtkWidget()
+{
+	return gtk_plug_new(instance->mWindow);
+}
+#endif
+
 NPError nsPluginInstance::SetWindow(NPWindow* aWindow)
 {
 	if(aWindow == NULL)
@@ -379,36 +466,36 @@ NPError nsPluginInstance::SetWindow(NPWindow* aWindow)
 
 	mX = aWindow->x;
 	mY = aWindow->y;
-	mWidth = aWindow->width;
-	mHeight = aWindow->height;
-	if(mHeight==0 || mHeight==0)
+	uint32_t width = aWindow->width;
+	uint32_t height = aWindow->height;
+	if(width==0 || height==0)
 	{
 		LOG(LOG_ERROR,_("No size in SetWindow"));
 		return NPERR_GENERIC_ERROR;
 	}
-	if (mWindow == (Window) aWindow->window)
+	if (mWindow == (NativeWindow) aWindow->window)
 	{
 		// The page with the plugin is being resized.
 		// Save any UI information because the next time
 		// around expect a SetWindow with a new window id.
 		LOG(LOG_ERROR,"Resize not supported");
+		return NPERR_NO_ERROR;
 	}
-	else
-	{
-		assert(mWindow==0);
-		mWindow = (Window) aWindow->window;
-		NPSetWindowCallbackStruct *ws_info = (NPSetWindowCallbackStruct *)aWindow->ws_info;
-		mDisplay = ws_info->display;
-		mVisual = ws_info->visual;
-		mDepth = ws_info->depth;
-		mColormap = ws_info->colormap;
+	assert(mWindow==0);
 
-		VisualID visual=XVisualIDFromVisual(mVisual);
-		PluginEngineData* e= new PluginEngineData(this, mDisplay, visual, mWindow, mWidth, mHeight);
-		LOG(LOG_INFO,"X Window " << hex << mWindow << dec << " Width: " << mWidth << " Height: " << mHeight);
-		m_sys->setParamsAndEngine(e, false);
-	}
-	//draw();
+	PluginEngineData* e = new PluginEngineData(this, width, height);
+	mWindow = (NativeWindow) aWindow->window;
+
+	LOG(LOG_INFO,"From Browser: Window " << mWindow << " Width: " << width << " Height: " << height);
+
+#ifndef _WIN32
+	NPSetWindowCallbackStruct *ws_info = (NPSetWindowCallbackStruct *)aWindow->ws_info;
+	//mDisplay = ws_info->display;
+	//mDepth = ws_info->depth;
+	//mColormap = ws_info->colormap;
+	e->visual = XVisualIDFromVisual(ws_info->visual);
+#endif
+	m_sys->setParamsAndEngine(e, false);
 	return NPERR_NO_ERROR;
 }
 
@@ -471,7 +558,7 @@ NPError nsPluginInstance::NewStream(NPMIMEType type, NPStream* stream, NPBool se
 {
 	NPDownloader* dl=static_cast<NPDownloader*>(stream->notifyData);
 	LOG(LOG_INFO,_("Newstream for ") << stream->url);
-	sys=m_sys;
+	setTLSSys( m_sys );
 	if(dl)
 	{
 		//Check if async destructin of this downloader has been requested
@@ -527,8 +614,8 @@ NPError nsPluginInstance::NewStream(NPMIMEType type, NPStream* stream, NPBool se
 	}
 	//The downloader is set as the private data for this stream
 	stream->pdata=dl;
-	sys=NULL;
-	return NPERR_NO_ERROR; 
+	setTLSSys( NULL );
+	return NPERR_NO_ERROR;
 }
 
 void nsPluginInstance::StreamAsFile(NPStream* stream, const char* fname)
@@ -549,7 +636,7 @@ int32_t nsPluginInstance::Write(NPStream *stream, int32_t offset, int32_t len, v
 {
 	if(stream->pdata)
 	{
-		sys=m_sys;
+		setTLSSys( m_sys );
 		NPDownloader* dl=static_cast<NPDownloader*>(stream->pdata);
 
 		//Check if async destructin of this downloader has been requested
@@ -564,7 +651,7 @@ int32_t nsPluginInstance::Write(NPStream *stream, int32_t offset, int32_t len, v
 		if(dl->hasFailed())
 			return -1;
 		dl->append((uint8_t*)buffer,len);
-		sys=NULL;
+		setTLSSys( NULL );
 		return len;
 	}
 	else
@@ -573,7 +660,7 @@ int32_t nsPluginInstance::Write(NPStream *stream, int32_t offset, int32_t len, v
 
 NPError nsPluginInstance::DestroyStream(NPStream *stream, NPError reason)
 {
-	sys=m_sys;
+	setTLSSys(m_sys);
 	NPDownloader* dl=static_cast<NPDownloader*>(stream->pdata);
 	assert(dl);
 	//Check if async destructin of this downloader has been requested
@@ -591,7 +678,7 @@ NPError nsPluginInstance::DestroyStream(NPStream *stream, NPError reason)
 	}
 	//Notify our downloader of what happened
 	URLNotify(stream->url, reason, stream->pdata);
-	sys=NULL;
+	setTLSSys(NULL);
 	return NPERR_NO_ERROR;
 }
 
@@ -617,23 +704,24 @@ void nsPluginInstance::URLNotify(const char* url, NPReason reason, void* notifyD
 	}
 }
 
-PluginEngineData::PluginEngineData(nsPluginInstance* i, Display* d, VisualID v, Window win, int w, int h):
-	EngineData(d,v,win,w,h),instance(i)
-{
-}
-
-void PluginEngineData::setupMainThreadCallback(lightspark::ls_callback_t func, void* arg)
-{
-	NPN_PluginThreadAsyncCall(instance->mInstance, func, arg);
-}
-
 void PluginEngineData::stopMainDownload()
 {
 	if(instance->mainDownloader)
 		instance->mainDownloader->stop();
 }
 
-bool PluginEngineData::isSizable() const
+NativeWindow PluginEngineData::getWindowForGnash()
 {
-	return false;
+	return instance->mWindow;
 }
+
+#ifdef _WIN32
+/* Setup for getExectuablePath() */
+extern HINSTANCE g_hinstance;
+extern "C"
+BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
+{
+	g_hinstance = hinstDLL;
+	return TRUE;
+}
+#endif
