@@ -43,7 +43,13 @@ class InteractiveObject;
 class Downloader;
 class AccessibilityProperties;
 
-class DisplayObject: public EventDispatcher
+class IBitmapDrawable
+{
+public:
+	static void linkTraits(Class_base* c);
+};
+
+class DisplayObject: public EventDispatcher, public IBitmapDrawable
 {
 friend class TokenContainer;
 friend std::ostream& operator<<(std::ostream& s, const DisplayObject& r);
@@ -371,6 +377,7 @@ public:
 	ASFUNCTION(lineStyle);
 	ASFUNCTION(beginFill);
 	ASFUNCTION(beginGradientFill);
+	ASFUNCTION(beginBitmapFill);
 	ASFUNCTION(endFill);
 	ASFUNCTION(drawRect);
 	ASFUNCTION(drawRoundRect);
@@ -394,9 +401,9 @@ protected:
 	_NR<InteractiveObject> hitTestImpl(_NR<InteractiveObject> last, number_t x, number_t y, DisplayObject::HIT_TYPE type)
 		{ return TokenContainer::hitTestImpl(last,x,y, type); }
 public:
-	Shape():TokenContainer(this), graphics(NULL) {}
+	Shape():TokenContainer(this), graphics() {}
 	Shape(const std::vector<GeomToken>& tokens, float scaling)
-		: TokenContainer(this, tokens, scaling), graphics(NULL) {}
+		: TokenContainer(this, tokens, scaling), graphics() {}
 	void finalize();
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
@@ -434,6 +441,7 @@ private:
 	LOAD_STATUS loadStatus;
 	Spinlock spinlock;
 public:
+	ASPROPERTY_GETTER(_NR<ASObject>,parameters);
 	LoaderInfo():bytesLoaded(0),bytesTotal(0),sharedEvents(NullRef),loader(NullRef),loadStatus(STARTED) {}
 	LoaderInfo(_R<Loader> l):bytesLoaded(0),bytesTotal(0),sharedEvents(NullRef),loader(l),loadStatus(STARTED) {}
 	void finalize();
@@ -742,21 +750,39 @@ public:
 	IntSize(uint32_t w, uint32_t h):width(h),height(h){}
 };
 
+class BitmapData: public ASObject, public IBitmapDrawable
+{
+CLASSBUILDABLE(BitmapData);
+private:
+	static void sinit(Class_base* c);
+	uint8_t* getData() { return data; }
+	int getWidth() { return width; }
+	int getHeight() { return height; }
+public:
+	BitmapData() : data(NULL), dataSize(0), width(0), height(0) {}
+	~BitmapData();
+	/* the bitmaps data in cairo's internal representation */
+	uint8_t* data;
+	size_t dataSize;
+	ASPROPERTY_GETTER(int32_t, width);
+	ASPROPERTY_GETTER(int32_t, height);
+	ASFUNCTION(draw);
+	bool fromRGB(uint8_t* rgb, uint32_t width, uint32_t height, bool hasAlpha);
+	bool fromJPEG(uint8_t* data, int len);
+	bool fromJPEG(std::istream& s);
+};
+
 class Bitmap: public DisplayObject, public TokenContainer
 {
 friend class CairoTokenRenderer;
 protected:
-	bool fromRGB(uint8_t* rgb, uint32_t width, uint32_t height, bool hasAlpha);
-	bool fromJPEG( uint8_t* data, int len);
-	bool fromJPEG(std::istream& s);
-	IntSize size;
-	/* the bitmaps data in cairo's internal representation */
-	uint8_t* data;
 	void renderImpl(bool maskEnabled, number_t t1, number_t t2, number_t t3, number_t t4) const
 		{ TokenContainer::renderImpl(maskEnabled,t1,t2,t3,t4); }
 public:
-	Bitmap() : TokenContainer(this), size(0,0), data(NULL) {}
-	Bitmap(std::istream *s, FILE_TYPE type=FT_UNKNOWN);
+	ASPROPERTY_GETTER(_NR<BitmapData>,bitmapData);
+	/* Call this after updating any member of 'data' */
+	void updatedData();
+	Bitmap(std::istream *s = NULL, FILE_TYPE type=FT_UNKNOWN);
 	static void sinit(Class_base* c);
 	bool boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const;
 	_NR<InteractiveObject> hitTestImpl(_NR<InteractiveObject> last, number_t x, number_t y, DisplayObject::HIT_TYPE type);
