@@ -65,7 +65,7 @@ void Event::finalize()
 void Event::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->super=Class<ASObject>::getRef();
+	c->setSuper(Class<ASObject>::getRef());
 
 	c->setVariableByQName("ENTER_FRAME","",Class<ASString>::getInstanceS("enterFrame"),DECLARED_TRAIT);
 	c->setVariableByQName("RENDER","",Class<ASString>::getInstanceS("render"),DECLARED_TRAIT);
@@ -172,7 +172,7 @@ ASFUNCTIONBODY(Event,clone)
 void EventPhase::sinit(Class_base* c)
 {
 	c->setConstructor(NULL);
-	c->super=Class<ASObject>::getRef();
+	c->setSuper(Class<ASObject>::getRef());
 	c->setVariableByQName("CAPTURING_PHASE","",abstract_i(CAPTURING_PHASE),DECLARED_TRAIT);
 	c->setVariableByQName("BUBBLING_PHASE","",abstract_i(BUBBLING_PHASE),DECLARED_TRAIT);
 	c->setVariableByQName("AT_TARGET","",abstract_i(AT_TARGET),DECLARED_TRAIT);
@@ -185,7 +185,7 @@ FocusEvent::FocusEvent():Event("focusEvent")
 void FocusEvent::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->super=Class<Event>::getRef();
+	c->setSuper(Class<Event>::getRef());
 
 	c->setVariableByQName("FOCUS_IN","",Class<ASString>::getInstanceS("focusIn"),DECLARED_TRAIT);
 	c->setVariableByQName("FOCUS_OUT","",Class<ASString>::getInstanceS("focusOut"),DECLARED_TRAIT);
@@ -218,7 +218,7 @@ ProgressEvent::ProgressEvent(uint32_t loaded, uint32_t total):Event("progress",f
 void ProgressEvent::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->super=Class<Event>::getRef();
+	c->setSuper(Class<Event>::getRef());
 
 	c->setVariableByQName("PROGRESS","",Class<ASString>::getInstanceS("progress"),DECLARED_TRAIT);
 	c->setDeclaredMethodByQName("bytesLoaded","",Class<IFunction>::getFunction(_getBytesLoaded),GETTER_METHOD,true);
@@ -255,7 +255,7 @@ ASFUNCTIONBODY(ProgressEvent,_getBytesTotal)
 void TimerEvent::sinit(Class_base* c)
 {
 //	c->constructor=Class<IFunction>::getFunction(_constructor);
-	c->super=Class<Event>::getRef();
+	c->setSuper(Class<Event>::getRef());
 
 	c->setVariableByQName("TIMER","",Class<ASString>::getInstanceS("timer"),DECLARED_TRAIT);
 	c->setVariableByQName("TIMER_COMPLETE","",Class<ASString>::getInstanceS("timerComplete"),DECLARED_TRAIT);
@@ -264,7 +264,7 @@ void TimerEvent::sinit(Class_base* c)
 void MouseEvent::sinit(Class_base* c)
 {
 //	c->constructor=Class<IFunction>::getFunction(_constructor);
-	c->super=Class<Event>::getRef();
+	c->setSuper(Class<Event>::getRef());
 
 	c->setVariableByQName("CLICK","",Class<ASString>::getInstanceS("click"),DECLARED_TRAIT);
 	c->setVariableByQName("DOUBLE_CLICK","",Class<ASString>::getInstanceS("doubleClick"),DECLARED_TRAIT);
@@ -300,7 +300,7 @@ ASFUNCTIONBODY(MouseEvent,_setter_localX)
 	th->localX = val;
 	//Change StageXY if target!=NULL else don't do anything
 	//At this point, the target should be an InteractiveObject but check anyway
-	if((th->target != NULL)&&(th->target->getClass()->isSubClass(Class<InteractiveObject>::getClass())))
+	if(th->target &&(th->target->getClass()->isSubClass(Class<InteractiveObject>::getClass())))
 	{		
 		InteractiveObject* tar = static_cast<InteractiveObject*>((th->target).getPtr());
 		tar->localToGlobal(th->localX, th->localY, th->stageX, th->stageY);
@@ -317,7 +317,7 @@ ASFUNCTIONBODY(MouseEvent,_setter_localY)
 	th->localY = val;
 	//Change StageXY if target!=NULL else don't do anything	
 	//At this point, the target should be an InteractiveObject but check anyway
-	if((th->target != NULL)&&(th->target->getClass()->isSubClass(Class<InteractiveObject>::getClass())))	
+	if(th->target &&(th->target->getClass()->isSubClass(Class<InteractiveObject>::getClass())))
 	{		
 		InteractiveObject* tar = static_cast<InteractiveObject*>((th->target).getPtr());
 		tar->localToGlobal(th->localX, th->localY, th->stageX, th->stageY);
@@ -336,7 +336,7 @@ void MouseEvent::setTarget(_NR<ASObject> t)
 {
 	target = t;
 	//If t is NULL, it means MouseEvent is being reset
-	if(t == NULL)
+	if(!t)
 	{
 		localX = 0;
 		localY = 0;
@@ -359,12 +359,12 @@ IOErrorEvent::IOErrorEvent() : ErrorEvent("ioError")
 void IOErrorEvent::sinit(Class_base* c)
 {
 //	c->constructor=Class<IFunction>::getFunction(_constructor);
-	c->super=Class<Event>::getRef();
+	c->setSuper(Class<Event>::getRef());
 
 	c->setVariableByQName("IO_ERROR","",Class<ASString>::getInstanceS("ioError"),DECLARED_TRAIT);
 }
 
-EventDispatcher::EventDispatcher():handlersMutex("handlersMutex")
+EventDispatcher::EventDispatcher()
 {
 }
 
@@ -377,8 +377,8 @@ void EventDispatcher::finalize()
 void EventDispatcher::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->addImplementedInterface(Class<IEventDispatcher>::getClass());
-	c->super=Class<ASObject>::getRef();
+	c->addImplementedInterface(InterfaceClass<IEventDispatcher>::getClass());
+	c->setSuper(Class<ASObject>::getRef());
 
 	c->setDeclaredMethodByQName("addEventListener","",Class<IFunction>::getFunction(addEventListener),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("hasEventListener","",Class<IFunction>::getFunction(_hasEventListener),NORMAL_METHOD,true);
@@ -396,14 +396,15 @@ void EventDispatcher::dumpHandlers()
 {
 	std::map<tiny_string,list<listener> >::iterator it=handlers.begin();
 	for(;it!=handlers.end();++it)
-		std::cout << it->first << std::endl;
+		LOG(LOG_INFO, it->first);
 }
 
 ASFUNCTIONBODY(EventDispatcher,addEventListener)
 {
 	EventDispatcher* th=Class<EventDispatcher>::cast(obj);
 	if(args[0]->getObjectType()!=T_STRING || args[1]->getObjectType()!=T_FUNCTION)
-		throw RunTimeException("Type mismatch in EventDispatcher::addEventListener");
+		//throw RunTimeException("Type mismatch in EventDispatcher::addEventListener");
+		return NULL;
 
 	bool useCapture=false;
 	int32_t priority=0;
@@ -422,7 +423,7 @@ ASFUNCTIONBODY(EventDispatcher,addEventListener)
 				|| eventName=="frameConstructed") )
 	{
 		dispobj->incRef();
-		sys->registerFrameListener(_MR(dispobj));
+		getSys()->registerFrameListener(_MR(dispobj));
 	}
 
 	{
@@ -493,7 +494,7 @@ ASFUNCTIONBODY(EventDispatcher,removeEventListener)
 					&& !th->hasEventListener("frameConstructed")) )
 	{
 		dispobj->incRef();
-		sys->unregisterFrameListener(_MR(dispobj));
+		getSys()->unregisterFrameListener(_MR(dispobj));
 	}
 
 	return NULL;
@@ -565,7 +566,7 @@ void EventDispatcher::handleEvent(_R<Event> e)
 
 	//Create a temporary copy of the listeners, as the list can be modified during the calls
 	vector<listener> tmpListener(h->second.begin(),h->second.end());
-	l.unlock();
+	l.release();
 	//TODO: check, ok we should also bind the level
 	for(unsigned int i=0;i<tmpListener.size();i++)
 	{
@@ -610,7 +611,7 @@ NetStatusEvent::NetStatusEvent(const tiny_string& l, const tiny_string& c):Event
 void NetStatusEvent::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->super=Class<Event>::getRef();
+	c->setSuper(Class<Event>::getRef());
 
 	c->setVariableByQName("NET_STATUS","",Class<ASString>::getInstanceS("netStatus"),DECLARED_TRAIT);
 }
@@ -650,7 +651,7 @@ FullScreenEvent::FullScreenEvent():Event("fullScreenEvent")
 void FullScreenEvent::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->super=Class<Event>::getRef();
+	c->setSuper(Class<Event>::getRef());
 
 	c->setVariableByQName("FULL_SCREEN","",Class<ASString>::getInstanceS("fullScreen"),DECLARED_TRAIT);
 }
@@ -667,7 +668,7 @@ KeyboardEvent::KeyboardEvent():Event("keyboardEvent")
 void KeyboardEvent::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->super=Class<Event>::getRef();
+	c->setSuper(Class<Event>::getRef());
 
 	c->setVariableByQName("KEY_DOWN","",Class<ASString>::getInstanceS("keyDown"),DECLARED_TRAIT);
 	c->setVariableByQName("KEY_UP","",Class<ASString>::getInstanceS("keyUp"),DECLARED_TRAIT);
@@ -685,7 +686,7 @@ TextEvent::TextEvent(const tiny_string& t):Event(t)
 void TextEvent::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->super=Class<Event>::getRef();
+	c->setSuper(Class<Event>::getRef());
 
 	c->setVariableByQName("TEXT_INPUT","",Class<ASString>::getInstanceS("textInput"),DECLARED_TRAIT);
 }
@@ -703,7 +704,7 @@ ErrorEvent::ErrorEvent(const tiny_string& t, const std::string& e): TextEvent(t)
 void ErrorEvent::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->super=Class<TextEvent>::getRef();
+	c->setSuper(Class<TextEvent>::getRef());
 
 	c->setVariableByQName("ERROR","",Class<ASString>::getInstanceS("error"),DECLARED_TRAIT);
 }
@@ -721,7 +722,7 @@ SecurityErrorEvent::SecurityErrorEvent(const std::string& e):ErrorEvent("securit
 void SecurityErrorEvent::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->super=Class<ErrorEvent>::getRef();
+	c->setSuper(Class<ErrorEvent>::getRef());
 
 	c->setVariableByQName("SECURITY_ERROR","",Class<ASString>::getInstanceS("securityError"),DECLARED_TRAIT);
 }
@@ -739,7 +740,7 @@ AsyncErrorEvent::AsyncErrorEvent():ErrorEvent("asyncError")
 void AsyncErrorEvent::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->super=Class<ErrorEvent>::getRef();
+	c->setSuper(Class<ErrorEvent>::getRef());
 
 	c->setVariableByQName("ASYNC_ERROR","",Class<ASString>::getInstanceS("asyncError"),DECLARED_TRAIT);
 }
@@ -750,7 +751,7 @@ ASFUNCTIONBODY(AsyncErrorEvent,_constructor)
 	return NULL;
 }
 
-ABCContextInitEvent::ABCContextInitEvent(ABCContext* c):Event("ABCContextInitEvent"),context(c)
+ABCContextInitEvent::ABCContextInitEvent(ABCContext* c, bool l):Event("ABCContextInitEvent"),context(c),lazy(l)
 {
 }
 
@@ -761,15 +762,15 @@ ShutdownEvent::ShutdownEvent():Event("shutdownEvent")
 void HTTPStatusEvent::sinit(Class_base* c)
 {
 //	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->super=Class<Event>::getRef();
+	c->setSuper(Class<Event>::getRef());
 
 	c->setVariableByQName("HTTP_STATUS","",Class<ASString>::getInstanceS("httpStatus"),DECLARED_TRAIT);
 }
 
 FunctionEvent::FunctionEvent(_R<IFunction> _f, _NR<ASObject> _obj, ASObject** _args, uint32_t _numArgs, 
-		ASObject** _result, ASObject** _exception, _NR<SynchronizationEvent> _sync):
-		Event("FunctionEvent"),f(_f),obj(_obj),numArgs(_numArgs),
-		result(_result),exception(_exception),sync(_sync)
+		ASObject** _result, ASObject** _exception):
+		WaitableEvent("FunctionEvent"),f(_f),obj(_obj),numArgs(_numArgs),
+		result(_result),exception(_exception)
 {
 	args = new ASObject*[numArgs];
 	uint32_t i;
