@@ -78,16 +78,15 @@ public:
 class Video: public DisplayObject
 {
 private:
-	mutable sem_t mutex;
+	mutable Mutex mutex;
 	uint32_t width, height;
 	mutable uint32_t videoWidth, videoHeight;
 	bool initialized;
 	_NR<NetStream> netStream;
 public:
 	Video(uint32_t w=320, uint32_t h=240)
-		: width(w),height(h),videoWidth(0),videoHeight(0),initialized(false),netStream(NULL)
+		: width(w),height(h),videoWidth(0),videoHeight(0),initialized(false),netStream()
 	{
-		sem_init(&mutex,0,1);
 	}
 	void finalize();
 	~Video();
