@@ -18,12 +18,10 @@
 **************************************************************************/
 
 #include <string>
-#include <pthread.h>
 #include <algorithm>
 #include "scripting/abc.h"
 #include "scripting/flash/events/flashevents.h"
 #include "scripting/flash/utils/flashutils.h"
-#include "swf.h"
 #include "logger.h"
 #include "parsing/streams.h"
 #include "asobject.h"
@@ -34,6 +32,7 @@
 #include "backends/rendering.h"
 #include "backends/security.h"
 #include "backends/image.h"
+#include "backends/extscriptobject.h"
 
 #ifdef ENABLE_CURL
 #include <curl/curl.h>
@@ -47,14 +46,30 @@ extern "C" {
 }
 #endif
 
-#include <gdk/gdkx.h>
-
 using namespace std;
 using namespace lightspark;
 
-extern TLSDATA ParseThread* pt;
+static GStaticPrivate tls_system = G_STATIC_PRIVATE_INIT;
+SystemState* lightspark::getSys()
+{
+	SystemState* ret = (SystemState*)g_static_private_get(&tls_system);
+	return ret;
+}
 
-RootMovieClip::RootMovieClip(LoaderInfo* li, bool isSys):mutex("mutexRoot"),parsingIsFailed(false),frameRate(0),
+void lightspark::setTLSSys(SystemState* sys)
+{
+        g_static_private_set(&tls_system,sys,NULL);
+}
+
+static GStaticPrivate parse_thread_tls = G_STATIC_PRIVATE_INIT; /* TLS */
+ParseThread* lightspark::getParseThread()
+{
+	ParseThread* pt = (ParseThread*)g_static_private_get(&parse_thread_tls);
+	assert(pt);
+	return pt;
+}
+
+RootMovieClip::RootMovieClip(LoaderInfo* li, bool isSys):parsingIsFailed(false),frameRate(0),
 	toBind(false), finishedLoading(false)
 {
 	if(li)
@@ -147,19 +162,19 @@ void SystemState::staticDeinit()
 }
 
 SystemState::SystemState(ParseThread* parseThread, uint32_t fileSize):
-	RootMovieClip(NULL,true),renderRate(0),error(false),shutdown(false),
-	renderThread(NULL),inputThread(NULL),engineData(NULL),fileDumpAvailable(0),
-	waitingForDump(false),vmVersion(VMNONE),childPid(0),useGnashFallback(false),
-	parameters(NullRef),mutexFrameListeners("mutexFrameListeners"),
+	RootMovieClip(NULL,true),terminated(0),renderRate(0),error(false),shutdown(false),
+	renderThread(NULL),inputThread(NULL),engineData(NULL),mainThread(0),dumpedSWFPathAvailable(0),
+	vmVersion(VMNONE),childPid(0),
+	parameters(NullRef),
 	invalidateQueueHead(NullRef),invalidateQueueTail(NullRef),showProfilingData(false),
 	currentVm(NULL),useInterpreter(true),useJit(false),exitOnError(false),downloadManager(NULL),
 	extScriptObject(NULL),scaleMode(SHOW_ALL)
 {
-	cookiesFileName[0]=0;
-	//Create the thread pool
-	sys=this;
-	sem_init(&terminated,0,0);
+	cookiesFileName = NULL;
 
+	setTLSSys(this);
+
+	mainThread = Thread::self();
 	//Get starting time
 	if(parseThread) //ParseThread may be null in tightspark
 		parseThread->setRootMovie(this);
@@ -192,9 +207,7 @@ SystemState::SystemState(ParseThread* parseThread, uint32_t fileSize):
 void SystemState::setDownloadedPath(const tiny_string& p)
 {
 	dumpedSWFPath=p;
-	Locker l(mutex);
-	if(waitingForDump)
-		fileDumpAvailable.signal();
+	dumpedSWFPathAvailable.signal();
 }
 
 void SystemState::setCookies(const char* c)
@@ -202,6 +215,11 @@ void SystemState::setCookies(const char* c)
 	rawCookies=c;
 }
 
+const std::string& SystemState::getCookies()
+{
+	return rawCookies;
+}
+
 static int hexToInt(char c)
 {
 	if(c>='0' && c<='9')
@@ -216,8 +234,9 @@ static int hexToInt(char c)
 
 void SystemState::parseParametersFromFlashvars(const char* v)
 {
-	if(useGnashFallback) //Save a copy of the string
-		rawParameters=v;
+	//Save a copy of the string
+	rawParameters=v;
+
 	_R<ASObject> params=_MR(Class<ASObject>::getInstanceS());
 	//Add arguments to SystemState
 	string vars(v);
@@ -225,7 +244,7 @@ void SystemState::parseParametersFromFlashvars(const char* v)
 	char* pfile = getenv("LIGHTSPARK_PLUGIN_PARAMFILE");
         ofstream f;
 	if(pfile)
-		f.open(pfile);
+		f.open(pfile, ios::binary|ios::out);
 	while(cur<vars.size())
 	{
 		int n1=vars.find('=',cur);
@@ -283,7 +302,7 @@ void SystemState::parseParametersFromFlashvars(const char* v)
 
 void SystemState::parseParametersFromFile(const char* f)
 {
-	ifstream i(f);
+	ifstream i(f, ios::in|ios::binary);
 	if(!i)
 	{
 		LOG(LOG_ERROR,_("Parameters file not found"));
@@ -305,8 +324,7 @@ void SystemState::parseParametersFromFile(const char* f)
 void SystemState::setParameters(_R<ASObject> p)
 {
 	parameters=p;
-	p->incRef();
-	loaderInfo->setVariableByQName("parameters","",p.getPtr(), DECLARED_TRAIT);
+	loaderInfo->parameters = p;
 }
 
 _NR<ASObject> SystemState::getParameters() const
@@ -319,14 +337,15 @@ void SystemState::stopEngines()
 	if(threadPool)
 		threadPool->forceStop();
 	timerThread->wait();
+	/* first shutdown the vm, because it can use all the others */
+	if(currentVm)
+		currentVm->shutdown();
 	delete downloadManager;
 	downloadManager=NULL;
 	delete securityManager;
 	securityManager=NULL;
 	delete config;
 	config=NULL;
-	if(currentVm)
-		currentVm->shutdown();
 	delete threadPool;
 	threadPool=NULL;
 	//Now stop the managers
@@ -369,15 +388,28 @@ void SystemState::destroy()
 #ifdef PROFILING_SUPPORT
 	saveProfilingInformation();
 #endif
+	terminated.wait();
+	//Acquire the mutex to sure that the engines are not being started right now
+	Locker l(mutex);
+	renderThread->wait();
+	inputThread->wait();
+	if(currentVm)
+		currentVm->shutdown();
+
+	l.release();
+
 	//Kill our child process if any
 	if(childPid)
 	{
-		assert(childPid!=getpid());
+		LOG(LOG_INFO,"Terminating gnash...");
 		kill_child(childPid);
 	}
 	//Delete the temporary cookies file
-	if(cookiesFileName[0])
+	if(cookiesFileName)
+	{
 		unlink(cookiesFileName);
+		g_free(cookiesFileName);
+	}
 	assert(shutdown);
 
 	renderThread->stop();
@@ -393,6 +425,7 @@ void SystemState::destroy()
 		threadPool->forceStop();
 	stopEngines();
 
+	delete extScriptObject;
 	delete intervalManager;
 	//Finalize ourselves
 	finalize();
@@ -432,6 +465,7 @@ void SystemState::destroy()
 	//This deletes the {int,uint,number}_managers; therefore no Number/.. object may be
 	//decRef'ed after this line as it would cause a manager->put()
 	delete currentVm;
+	currentVm = NULL;
 
 	//Some objects needs to remove the jobs when destroyed so keep the timerThread until now
 	delete timerThread;
@@ -442,7 +476,10 @@ void SystemState::destroy()
 	delete inputThread;
 	inputThread=NULL;
 	delete engineData;
-	sem_destroy(&terminated);
+
+	for(auto it=profilingData.begin();it!=profilingData.end();it++)
+		delete *it;
+
 	this->decRef(); //free a reference we obtained by 'new SystemState'
 }
 
@@ -487,20 +524,15 @@ void SystemState::setShutdownFlag()
 	}
 	shutdown=true;
 
-	sem_post(&terminated);
+	terminated.signal();
 	if(standalone)
+	{
+		if(Thread::self() != mainThread)
+			gdk_threads_enter();
 		gtk_main_quit();
-}
-
-void SystemState::wait()
-{
-	sem_wait(&terminated);
-	//Acquire the mutex to sure that the engines are not being started right now
-	Locker l(mutex);
-	renderThread->wait();
-	inputThread->wait();
-	if(currentVm)
-		currentVm->shutdown();
+		if(Thread::self() != mainThread)
+			 gdk_threads_leave();
+	}
 }
 
 float SystemState::getRenderRate()
@@ -518,73 +550,49 @@ void SystemState::startRenderTicks()
 
 void SystemState::EngineCreator::execute()
 {
-	sys->createEngines();
+	getSys()->createEngines();
 }
 
 void SystemState::EngineCreator::threadAbort()
 {
-	sys->fileDumpAvailable.signal();
-	sys->getRenderThread()->forceInitialization();
+	getSys()->dumpedSWFPathAvailable.signal();
+	getSys()->getRenderThread()->forceInitialization();
 }
 
-#ifndef GNASH_PATH
-#error No GNASH_PATH defined
-#endif
-
-void SystemState::enableGnashFallback()
+/*
+ * This is run from the gtk main thread.
+ * gtk/gdk functions may only be called from within that
+ * gtk main thread for portability. Win32 does not support
+ * gtk/gdk calls from other threads!
+ */
+void SystemState::delayedCreation()
 {
-	//Check if the gnash standalone executable is available
-	ifstream f(GNASH_PATH);
-	if(f)
-		useGnashFallback=true;
-	f.close();
-}
+	gdk_threads_enter();
+
+	int32_t reqWidth=getFrameSize().Xmax/20;
+	int32_t reqHeight=getFrameSize().Ymax/20;
+
+	engineData->showWindow(reqWidth, reqHeight);
 
-void SystemState::delayedCreation(SystemState* th)
-{
-	EngineData* d=th->engineData;
-	//Create a plug in the XEmbed window
-	GtkWidget* plug=gtk_plug_new(d->window);
-	if(d->isSizable())
-	{
-		int32_t reqWidth=th->getFrameSize().Xmax/20;
-		int32_t reqHeight=th->getFrameSize().Ymax/20;
-		if(th->standalone)
-			gtk_widget_set_size_request(plug, reqWidth, reqHeight);
-		d->width=reqWidth;
-		d->height=reqHeight;
-	}
-	d->container = plug;
-	//Realize the widget now, as we need the X window
-	gtk_widget_realize(plug);
-	//Show it now
-	gtk_widget_show(plug);
-	gtk_widget_map(plug);
-	if (th->standalone)
-	{
-		gtk_widget_set_can_focus(plug, true);
-		gtk_widget_grab_focus(plug);
-	}
-	d->window=GDK_WINDOW_XID(gtk_widget_get_window(plug));
-	XSync(d->display, False);
 	//The lock is needed to avoid thread creation/destruction races
-	Locker l(th->mutex);
-	if(th->shutdown)
+	Locker l(mutex);
+	if(shutdown)
 		return;
-	th->renderThread->start(th->engineData);
-	th->inputThread->start(th->engineData);
+	renderThread->start(engineData);
+	inputThread->start(engineData);
 	//If the render rate is known start the render ticks
-	if(th->renderRate)
-		th->startRenderTicks();
+	if(renderRate)
+		startRenderTicks();
+	gdk_threads_leave();
 }
 
-void SystemState::delayedStopping(SystemState* th)
+void SystemState::delayedStopping()
 {
-	sys=th;
+	setTLSSys(this);
 	//This is called from the plugin, also kill the stream
-	th->engineData->stopMainDownload();
-	th->stopEngines();
-	sys=NULL;
+	engineData->stopMainDownload();
+	stopEngines();
+	setTLSSys(NULL);
 }
 
 void SystemState::createEngines()
@@ -596,178 +604,29 @@ void SystemState::createEngines()
 		return;
 	}
 	//Check if we should fall back on gnash
-	if(useGnashFallback && vmVersion!=AVM2)
+	if(vmVersion!=AVM2)
 	{
-		if(dumpedSWFPath.len()==0) //The path is not known yet
-		{
-			waitingForDump=true;
-			l.unlock();
-			fileDumpAvailable.wait();
-			if(shutdown)
-				return;
-			l.lock();
-		}
-		LOG(LOG_INFO,_("Trying to invoke gnash!"));
-		//Dump the cookies to a temporary file
-		strcpy(cookiesFileName,"/tmp/lightsparkcookiesXXXXXX");
-		int file=mkstemp(cookiesFileName);
-		if(file!=-1)
-		{
-			std::string data("Set-Cookie: " + rawCookies);
-			ssize_t res;
-			size_t written = 0;
-			// Keep writing until everything we wanted to write actually got written
-			do
-			{
-				res = write(file, data.c_str()+written, data.size()-written);
-				if(res < 0)
-				{
-					LOG(LOG_ERROR, _("Error during writing of cookie file for Gnash"));
-					break;
-				}
-				written += res;
-			} while(written < data.size());
-			close(file);
-			setenv("GNASH_COOKIES_IN", cookiesFileName, 1);
-		}
-		else
-			cookiesFileName[0]=0;
-		sigset_t oldset;
-		sigset_t set;
-		sigfillset(&set);
-		//Blocks all signal to avoid terminating while forking with the browser signal handlers on
-		pthread_sigmask(SIG_SETMASK, &set, &oldset);
+		launchGnash();
 
-		// This will be used to pipe the SWF's data to Gnash's stdin
-		int gnashStdin[2];
-		if (pipe(gnashStdin) != 0)
-			LOG(LOG_ERROR,_("Pipe creation failed"));
-
-		childPid=fork();
-		if(childPid==-1)
-		{
-			//Restore handlers
-			pthread_sigmask(SIG_SETMASK, &oldset, NULL);
-			LOG(LOG_ERROR,_("Child process creation failed, lightspark continues"));
-			childPid=0;
-		}
-		else if(childPid==0) //Child process scope
-		{
-			// Close write end of Gnash's stdin pipe, we will only read
-			close(gnashStdin[1]);
-			// Point stdin to the read end of Gnash's stdin pipe
-			dup2(gnashStdin[0], fileno(stdin));
-			// Close the read end of the pipe
-			close(gnashStdin[0]);
-
-			//Allocate some buffers to store gnash arguments
-			char bufXid[32];
-			char bufWidth[32];
-			char bufHeight[32];
-			snprintf(bufXid,32,"%lu",engineData->window);
-			snprintf(bufWidth,32,"%u",engineData->width);
-			snprintf(bufHeight,32,"%u",engineData->height);
-			string params("FlashVars=");
-			params+=rawParameters;
-			char *const args[] =
-			{
-				strdup("gnash"), //argv[0]
-				strdup("-x"), //Xid
-				bufXid,
-				strdup("-j"), //Width
-				bufWidth,
-				strdup("-k"), //Height
-				bufHeight,
-				strdup("-u"), //SWF url
-				strdup(origin.getParsedURL().raw_buf()),
-				strdup("-P"), //SWF parameters
-				strdup(params.c_str()),
-				strdup("-vv"),
-				strdup("-"),
-				NULL
-			};
-
-			// Print out an informative message about how we are invoking Gnash
-			{
-				int i = 1;
-				std::string argsStr = "";
-				while(args[i] != NULL)
-				{
-					argsStr += " ";
-					argsStr += args[i];
-					i++;
-				}
-				cerr << "Invoking '" << GNASH_PATH << argsStr << " < " << dumpedSWFPath.raw_buf() << "'" << endl;
-			}
-
-			//Avoid calling browser signal handler during the short time between enabling signals and execve
-			sigaction(SIGTERM, NULL, NULL);
-			//Restore handlers
-			pthread_sigmask(SIG_SETMASK, &oldset, NULL);
-			execve(GNASH_PATH, args, environ);
-			//If we are here execve failed, print an error and die
-			cerr << _("Execve failed, content will not be rendered") << endl;
-			exit(0);
-		}
-		else //Parent process scope
-		{
-			// Pass the SWF's data to Gnash
-			{
-				// Close read end of stdin pipe, we will only write to it.
-				close(gnashStdin[0]);
-				// Open the SWF file
-				std::ifstream swfStream(dumpedSWFPath.raw_buf(), ios::binary);
-				// Read the SWF file and write it to Gnash's stdin
-				char data[1024];
-				std::streamsize written, ret;
-				bool stop = false;
-				while(!swfStream.eof() && !swfStream.fail() && !stop)
-				{
-					swfStream.read(data, 1024);
-					// Keep writing until everything we wanted to write actually got written
-					written = 0;
-					do
-					{
-						ret = write(gnashStdin[1], data+written, swfStream.gcount()-written);
-						if(ret < 0)
-						{
-							LOG(LOG_ERROR, _("Error during writing of SWF file to Gnash"));
-							stop = true;
-							break;
-						}
-						written += ret;
-					} while(written < swfStream.gcount());
-				}
-				// Close the write end of Gnash's stdin, signalling EOF to Gnash.
-				close(gnashStdin[1]);
-				// Close the SWF file
-				swfStream.close();
-			}
-
-			//Restore handlers
-			pthread_sigmask(SIG_SETMASK, &oldset, NULL);
-			//Engines should not be started, stop everything
-			l.unlock();
-			//We cannot stop the engines now, as this is inside a ThreadPool job
-			engineData->setupMainThreadCallback((ls_callback_t)delayedStopping, this);
-			return;
-		}
+		//Engines should not be started, stop everything
+		l.release();
+		//We cannot stop the engines now, as this is inside a ThreadPool job
+		//Running this in the Gtk thread is unnecessary, though. Any other thread
+		//would be okey.
+		engineData->runInGtkThread(sigc::mem_fun(this, &SystemState::delayedStopping));
+		return;
 	}
 
-	l.unlock();
+	l.release();
 	//The engines must be created in the context of the main thread
-	engineData->setupMainThreadCallback((ls_callback_t)delayedCreation, this);
+	engineData->runInGtkThread(sigc::mem_fun(this, &SystemState::delayedCreation));
 
 	renderThread->waitForInitialization();
 
 	// If the SWF file is AVM1 and Gnash fallback isn't enabled, just shut down.
-	if(vmVersion != AVM2)
-	{
-		LOG(LOG_INFO, "Unsupported flash file (AVM1), shutting down...");
-		setShutdownFlag();
-	}
 
-	l.lock();
+
+	l.acquire();
 	//As we lost the lock the shutdown procesure might have started
 	if(shutdown)
 		return;
@@ -775,6 +634,142 @@ void SystemState::createEngines()
 		currentVm->start();
 }
 
+void SystemState::launchGnash()
+{
+	if(config->getGnashPath().empty())
+	{
+		LOG(LOG_INFO, "Unsupported flash file (AVM1), and no gnash found");
+		setShutdownFlag();
+		return;
+	}
+
+	/* wait for dumpedSWFPath */
+	dumpedSWFPathAvailable.wait();
+	if(dumpedSWFPath.empty())
+		return;
+
+	LOG(LOG_INFO,_("Trying to invoke gnash!"));
+	//Dump the cookies to a temporary file
+	int file=g_file_open_tmp("lightsparkcookiesXXXXXX",&cookiesFileName,NULL);
+	if(file!=-1)
+	{
+		std::string data("Set-Cookie: " + rawCookies);
+		ssize_t res;
+		size_t written = 0;
+		// Keep writing until everything we wanted to write actually got written
+		do
+		{
+			res = write(file, data.c_str()+written, data.size()-written);
+			if(res < 0)
+			{
+				LOG(LOG_ERROR, _("Error during writing of cookie file for Gnash"));
+				break;
+			}
+			written += res;
+		} while(written < data.size());
+		close(file);
+		g_setenv("GNASH_COOKIES_IN", cookiesFileName, 1);
+	}
+	else
+	{
+		LOG(LOG_ERROR,"Failed to create temporary coockie for gnash");
+	}
+
+
+	//Allocate some buffers to store gnash arguments
+	char bufXid[32];
+	char bufWidth[32];
+	char bufHeight[32];
+	snprintf(bufXid,32,"%lu",(long unsigned)engineData->getWindowForGnash());
+	/* Use swf dimensions in standalone mode and window dimensions in plugin mode */
+	snprintf(bufWidth,32,"%u",standalone ? getFrameSize().Xmax/20 : engineData->width);
+	snprintf(bufHeight,32,"%u",standalone ? getFrameSize().Ymax/20 : engineData->height);
+	string params("FlashVars=");
+	params+=rawParameters;
+	/* TODO: pass -F hostFD to assist in loading urls */
+	char* args[] =
+	{
+		strdup(config->getGnashPath().c_str()),
+		strdup("-x"), //Xid
+		bufXid,
+		strdup("-j"), //Width
+		bufWidth,
+		strdup("-k"), //Height
+		bufHeight,
+		strdup("-u"), //SWF url
+		strdup(origin.getParsedURL().raw_buf()),
+		strdup("-P"), //SWF parameters
+		strdup(params.c_str()),
+		strdup("-vv"),
+		strdup("-"),
+		NULL
+	};
+
+	// Print out an informative message about how we are invoking Gnash
+	int i = 1;
+	std::string argsStr = args[0];
+	while(args[i] != NULL)
+	{
+		argsStr += " ";
+		argsStr += args[i];
+		i++;
+	}
+	LOG(LOG_INFO, "Invoking '" << argsStr << " < " << dumpedSWFPath.raw_buf() << "'");
+
+	int gnash_stdin;
+
+	/* Unfortunately, g_spawn_async_with_pipes does not work under win32. First, it needs
+	 * an additional helper 'gspawn-helper-console.exe' and second, it crashes with a buffer overflow
+	 * when the plugin is run in ipc mode.
+	 */
+#if _WIN32
+	//TODO: escape argumetns, and spaces in filename
+	childPid = compat_spawn(args, &gnash_stdin);
+	if(!childPid)
+	{
+		LOG(LOG_ERROR,"Spawning gnash failed!");
+		return;
+	}
+#else
+	GError* errmsg;
+	if(!g_spawn_async_with_pipes(NULL, args, NULL, (GSpawnFlags)0, NULL, NULL, &childPid,
+			&gnash_stdin, NULL, NULL, &errmsg))
+	{
+		LOG(LOG_ERROR,"Spawning gnash failed: " << errmsg->message);
+		return;
+	}
+#endif
+
+	// Open the SWF file
+	std::ifstream swfStream(dumpedSWFPath.raw_buf(), ios::in|ios::binary);
+	// Read the SWF file and write it to Gnash's stdin
+	char data[1024];
+	std::streamsize written, ret;
+	bool stop = false;
+	while(!swfStream.eof() && !swfStream.fail() && !stop)
+	{
+		swfStream.read(data, 1024);
+		// Keep writing until everything we wanted to write actually got written
+		written = 0;
+		do
+		{
+			ret = write(gnash_stdin, data+written, swfStream.gcount()-written);
+			if(ret < 0)
+			{
+				LOG(LOG_ERROR, _("Error during writing of SWF file to Gnash"));
+				stop = true;
+				break;
+			}
+			written += ret;
+		} while(written < swfStream.gcount());
+	}
+	// Close the write end of Gnash's stdin, signalling EOF to Gnash.
+	close(gnash_stdin);
+	// Close the SWF file
+	swfStream.close();
+}
+
+
 void SystemState::needsAVM2(bool n)
 {
 	Locker l(mutex);
@@ -828,16 +823,16 @@ void SystemState::addWait(uint32_t waitTime, ITickJob* job)
 	timerThread->addWait(waitTime,job);
 }
 
-bool SystemState::removeJob(ITickJob* job)
+void SystemState::removeJob(ITickJob* job)
 {
-	return timerThread->removeJob(job);
+	timerThread->removeJob(job);
 }
 
 ThreadProfile* SystemState::allocateProfiler(const lightspark::RGB& color)
 {
 	SpinlockLocker l(profileDataSpinlock);
-	profilingData.push_back(ThreadProfile(color,100));
-	ThreadProfile* ret=&profilingData.back();
+	profilingData.push_back(new ThreadProfile(color,100));
+	ThreadProfile* ret=profilingData.back();
 	return ret;
 }
 
@@ -847,7 +842,7 @@ void SystemState::addToInvalidateQueue(_R<DisplayObject> d)
 	//Check if the object is already in the queue
 	if(!d->invalidateQueueNext.isNull() || d==invalidateQueueTail)
 		return;
-	if(invalidateQueueHead==NULL)
+	if(!invalidateQueueHead)
 	{
 		invalidateQueueHead=invalidateQueueTail=d;
 		//This is the first object added to the invalidation queue
@@ -936,7 +931,7 @@ void ThreadProfile::plot(uint32_t maxTime, cairo_t *cr)
 		return;
 
 	Locker locker(mutex);
-	RECT size=sys->getFrameSize();
+	RECT size=getSys()->getFrameSize();
 	int width=size.Xmax/20;
 	int height=size.Ymax/20;
 	
@@ -981,7 +976,7 @@ void ThreadProfile::plot(uint32_t maxTime, cairo_t *cr)
 		{
 			//New tag, flush the old one if present
 			if(curTag)
-				rt->renderText(cr, curTag->c_str(),curTagX,imax(curTagY-curTagH,0));
+				getRenderThread()->renderText(cr, curTag->c_str(),curTagX,imax(curTagY-curTagH,0));
 			//Measure tag
 			cairo_text_extents (cr, data[i].tag.c_str(), &te);
 			curTagLen=te.width;
@@ -997,7 +992,7 @@ void ThreadProfile::plot(uint32_t maxTime, cairo_t *cr)
 			else
 			{
 				//Tag is before this sample
-				rt->renderText(cr, curTag->c_str(), curTagX, imax(curTagY-curTagH,0));
+				getRenderThread()->renderText(cr, curTag->c_str(), curTagX, imax(curTagY-curTagH,0));
 				curTag=NULL;
 			}
 		}
@@ -1005,7 +1000,7 @@ void ThreadProfile::plot(uint32_t maxTime, cairo_t *cr)
 }
 
 ParseThread::ParseThread(istream& in, Loader *_loader, tiny_string srcurl)
-  : version(0),useAVM2(false),useNetwork(false),
+  : version(0),
     f(in),zlibFilter(NULL),backend(NULL),loader(_loader),
     parsedObject(NullRef),url(srcurl),fileType(FT_UNKNOWN)
 {
@@ -1013,7 +1008,7 @@ ParseThread::ParseThread(istream& in, Loader *_loader, tiny_string srcurl)
 }
 
 ParseThread::ParseThread(std::istream& in, RootMovieClip *root)
-  : version(0),useAVM2(false),useNetwork(false),
+  : version(0),
     f(in),zlibFilter(NULL),backend(NULL),loader(NULL),
     parsedObject(NullRef),url(),fileType(FT_UNKNOWN)
 {
@@ -1078,14 +1073,14 @@ void ParseThread::parseSWFHeader(RootMovieClip *root, UI8 ver)
 	LOG(LOG_INFO,_("FrameRate ") << frameRate);
 	root->setFrameRate(frameRate);
 	//TODO: setting render rate should be done when the clip is added to the displaylist
-	sys->setRenderRate(frameRate);
+	getSys()->setRenderRate(frameRate);
 	root->setFrameSize(FrameSize);
 	root->totalFrames_unreliable = FrameCount;
 }
 
 void ParseThread::execute()
 {
-	pt=this;
+	g_static_private_set(&parse_thread_tls,this,NULL);
 	try
 	{
 		UI8 Signature[4];
@@ -1107,13 +1102,12 @@ void ParseThread::execute()
 	catch(LightsparkException& e)
 	{
 		LOG(LOG_ERROR,_("Exception in ParseThread ") << e.cause);
-		sys->setError(e.cause);
+		getSys()->setError(e.cause);
 	}
 	catch(std::exception& e)
 	{
 		LOG(LOG_ERROR,_("Stream exception in ParseThread ") << e.what());
 	}
-	pt=NULL;
 }
 
 void ParseThread::parseSWF(UI8 ver)
@@ -1124,14 +1118,42 @@ void ParseThread::parseSWF(UI8 ver)
 	try
 	{
 		parseSWFHeader(root, ver);
+		if(root->version < 9)
+		{
+			LOG(LOG_INFO,"SWF version " << root->version << " is not handled by lightspark, falling back to gnash (if available)");
+			//Enable flash fallback
+			getSys()->needsAVM2(false);
+			return; /* no more parsing necessary, handled by fallback */
+		}
 
-		//Create a top level TagFactory
 		TagFactory factory(f, true);
+		_NR<Tag> tag=factory.readTag();
+
+		FileAttributesTag* fat = dynamic_cast<FileAttributesTag*>(tag.getPtr());
+		if(!fat)
+		{
+			LOG(LOG_ERROR,"Invalid SWF - First tag must be a FileAttributesTag!");
+			return;
+		}
+		//Check if this clip is the main clip then honour its FileAttributesTag
+		if(root == getSys())
+		{
+			getSys()->needsAVM2(fat->ActionScript3);
+			if(!fat->ActionScript3)
+				return; /* no more parsing necessary, handled by fallback */
+			if(fat->UseNetwork
+			&& getSys()->securityManager->getSandboxType() == SecurityManager::LOCAL_WITH_FILE)
+			{
+				getSys()->securityManager->setSandboxType(SecurityManager::LOCAL_WITH_NETWORK);
+				LOG(LOG_INFO, _("Switched to local-with-networking sandbox by FileAttributesTag"));
+			}
+		}
+
 		bool done=false;
 		bool empty=true;
 		while(!done)
 		{
-			_NR<Tag> tag=factory.readTag();
+			tag=factory.readTag();
 			switch(tag->getType())
 			{
 				case END_TAG:
@@ -1188,7 +1210,7 @@ void ParseThread::parseSWF(UI8 ver)
 					//Not yet implemented tag, ignore it
 					break;
 			}
-			if(sys->shouldTerminate() || aborting)
+			if(getSys()->shouldTerminate() || aborting)
 				break;
 		}
 	}
@@ -1197,6 +1219,7 @@ void ParseThread::parseSWF(UI8 ver)
 		root->parsingFailed();
 		throw;
 	}
+	LOG(LOG_INFO,_("End of parsing"));
 }
 
 void ParseThread::parseBitmap()
@@ -1226,7 +1249,7 @@ void ParseThread::setRootMovie(RootMovieClip *root)
 RootMovieClip *ParseThread::getRootMovie()
 {
 	objectSpinlock.lock();
-	RootMovieClip *root=Class<RootMovieClip>::dyncast(parsedObject.getPtr());
+	RootMovieClip *root=dynamic_cast<RootMovieClip*>(parsedObject.getPtr());
 	objectSpinlock.unlock();
 	if(root)
 		return root;
@@ -1237,7 +1260,7 @@ RootMovieClip *ParseThread::getRootMovie()
 		objectSpinlock.lock();
 		parsedObject=_MNR(root);
 		objectSpinlock.unlock();
-		if(url.len()>0)
+		if(!url.empty())
 			root->setOrigin(url, "");
 		return root;
 	}
@@ -1289,10 +1312,11 @@ void RootMovieClip::commitFrame(bool another)
 	setFramesLoaded(frames.size());
 
 	if(another)
-		frames.emplace_back();
+		frames.push_back(Frame());
 
 	if(getFramesLoaded()==1 && frameRate!=0)
 	{
+		SystemState* sys = getSys();
 		if(this==sys)
 		{
 			/* now the frameRate is available and all SymbolClass tags have created their classes */
@@ -1416,11 +1440,11 @@ void SystemState::tick()
 {
 	{
 		SpinlockLocker l(profileDataSpinlock);
-		list<ThreadProfile>::iterator it=profilingData.begin();
+		list<ThreadProfile*>::iterator it=profilingData.begin();
 		for(;it!=profilingData.end();++it)
-			it->tick();
+			(*it)->tick();
 	}
-	if(sys->currentVm==NULL)
+	if(getSys()->currentVm==NULL)
 		return;
 	/* See http://www.senocular.com/flash/tutorials/orderofoperations/
 	 * for the description of steps.
@@ -1442,7 +1466,7 @@ void SystemState::tick()
 	/* Step 3: create legacy objects, which are new in this frame (top-down),
 	 * run their constructors (bottom-up)
 	 * and their frameScripts (Step 5) (bottom-up) */
-	sys->currentVm->addEvent(NullRef, _MR(new InitFrameEvent()));
+	getSys()->currentVm->addEvent(NullRef, _MR(new InitFrameEvent()));
 
 	/* Step 4: dispatch frameConstructed events */
 	/* (TODO: should be run between step 3 and 5 */
@@ -1471,7 +1495,7 @@ void SystemState::tick()
 
 	/* Step 0: Set current frame number to the next frame */
 	_R<AdvanceFrameEvent> advFrame = _MR(new AdvanceFrameEvent());
-	if(sys->currentVm->addEvent(NullRef, advFrame))
+	if(getSys()->currentVm->addEvent(NullRef, advFrame))
 		advFrame->done.wait();
 }
 
@@ -1510,6 +1534,6 @@ void RootMovieClip::advanceFrame()
 void RootMovieClip::constructionComplete()
 {
 	MovieClip::constructionComplete();
-	if(this==sys)
+	if(this==getSys())
 		loaderInfo->sendInit();
 }
